/*
 * @note
 * Copyright(C) NXP Semiconductors, 2020
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include <stdlib.h>
#include "board.h"
#include "ntag_driver.h"
#include "ntag_bridge.h"
#include "boardselection.h"

NTAG_HANDLE_T ntag;
NTAG_DEVICE_t nDevice;
phbalReg_Type_t sbalReg;

bool globalBool = false;

int main(void)
{
	ntag = &nDevice;
	uint8_t buffer[4] = {0};
	uint8_t buffer_w[4] = {1,4,2,3};
	uint8_t buffer_e[4] = {0};

	/* Board Initialization */
	SystemCoreClockUpdate();
	Board_Init();

	/* Initialize I2C */
	phbalReg_Init(&sbalReg, sizeof(sbalReg));

	/* Initialize NTAG Middleware. */
	NTAG_InitDevice(ntag, &sbalReg);

	/* Sets the Arbiter mode to normal */
	NTAG_WriteRegister(ntag, NTAG_MEM_OFFSET_TAG_CONFIG0_REG, 0x0C00, 0x0000);

	/* Continuous Loop, which exits on Field detection Interrupt */
	while(1)
	{
		/* globalBool is set to true by the Interrupt routine for the ED Pin */
		if(globalBool == true)
		{
			break;
		}
		/* Set the RF field present interrupt to detect when the phone has tapped the antenna.*/
		NTAG_ClearED(ntag);
		NTAG_WaitForEvent( ntag, NTAG_EVENT_RF_FIELD_PRESENT_INTERRUPT, 5, 1);
	}


	/* Reads Block 5 and stores the values in buffer. Expected Value: 0,0,0,0*/
	NTAG_ReadBytes(ntag, 0x05, buffer, 4);
	/* Writes Block 5 the values from buffer_w {1,4,2,3} */
	NTAG_WriteBytes(ntag, 0x05, buffer_w, 4);
	/* Reads Block 5 and stores the values in buffer. Expected Value: 1,4,2,3*/
	NTAG_ReadBytes(ntag, 0x05, buffer, 4);
	/* Writes Block 5 the values from buffer_e {0,0,0,0} */
	NTAG_WriteBytes(ntag, 0x05, buffer_e,4);
	/* Reads Block 5 and stores the values in buffer. Expected Value: 0,0,0,0*/
	NTAG_ReadBytes(ntag, 0x05, buffer, 4);


	Chip_I2C_DeInit(I2C0);

	return 0;
}

/* Interrupt routine for ED Pin interrupt */
void FLEX_INT0_IRQHandler(void)
{
	globalBool = true;

	phDriver_PinClearIntStatus(PHDRIVER_IRQGROUP);

	/* Call  callback. */
	if(ntag->ISR_EventCallback != NULL)
	{
		ntag->ISR_EventCallback();
	}
}
