/*
*         Copyright (c), NXP Semiconductors Bangalore / India
*
*                     (C)NXP Semiconductors
*       All rights are reserved. Reproduction in whole or in part is
*      prohibited without the written consent of the copyright owner.
*  NXP reserves the right to make changes without notice at any time.
* NXP makes no warranty, expressed, implied or statutory, including but
* not limited to any implied warranty of merchantability or fitness for any
*particular purpose, or that the use will not infringe any third party patent,
* copyright or trademark. NXP must not be liable for any loss or damage
*                          arising from its use.
*/

/** \file
* Generic phDriver Component of Reader Library Framework.
* $Author$
* $Revision$
* $Date$
*
* History:
*  RS: Generated 24. Jan 2017
*
*/

#ifndef BOARD_FRDM_KW41Z_H_
#define BOARD_FRDM_KW41Z_H_


#define PHDRIVER_I2CSLAVE_ADDRESS       0x54  /*!< Address of the intended slave, for NTAG5 it is 0x55*/

#define GPIO_PORT_A         0    /* Same macro for GPIOA/PORTA */
#define GPIO_PORT_B         1    /* Same macro for GPIOB/PORTB */
#define GPIO_PORT_C         2    /* Same macro for GPIOC/PORTC */
#define GPIO_PORT_D         3    /* Same macro for GPIOD/PORTD */
#define GPIO_PORT_E         4    /* Same macro for GPIOE/PORTE */

#define PIN6_IDX                         6u   /*!< Pin number for pin 6 in a port */
#define PIN7_IDX                         7u   /*!< Pin number for pin 7 in a port */

/******************************************************************
 * LPUART configurations
 ******************************************************************/
#define SOPT5_LPUART0RXSRC_LPUART_RX  0x00u   /*!< LPUART0 Receive Data Source Select: LPUART_RX pin */
#define BOARD_DEBUG_UART_BAUDRATE 115200
#define DEMO_LPUART LPUART0
#define DEMO_LPUART_CLKSRC kCLOCK_CoreSysClk
#define DEMO_LPUART_CLK_FREQ CLOCK_GetFreq(kCLOCK_CoreSysClk)


/******************************************************************
 * Board Pin/Gpio configurations
 ******************************************************************/
/* Pin configuration format : Its a 32 bit format where every byte represents a field as shown below.
 * | Byte3 | Byte2 | Byte1      | Byte0 |
 * |  --   |  --   | GPIO/PORT  | PIN   |
 * */
#define PHDRIVER_PIN_RESET  ((GPIO_PORT_A << 8) | 13)  /**< Reset pin, Pin13, GPIOA, PORTA */
#define PHDRIVER_PIN_IRQ    ((GPIO_PORT_C << 8) | 17)   /**< IRQ pin, Pin17, GPIOC, PORTC */
#define PHDRIVER_PIN_GPIOC1    ((GPIO_PORT_C << 8) | 1)   /**< IRQ pin, Pin1, GPIOC, PORTC RED LED */
#define PHDRIVER_PIN_GPIOA19    ((GPIO_PORT_A << 8) | 19)   /**< IRQ pin, Pin19, GPIOA, PORTA Green */
#define PHDRIVER_PIN_GPIOA18    ((GPIO_PORT_A << 8) | 18)   /**< IRQ pin, Pin18, GPIOA, PORTA Blue */

/******************************************************************
 * PIN Pull-Up/Pull-Down configurations.
 ******************************************************************/
#define PHDRIVER_PIN_RESET_PULL_CFG    PH_DRIVER_PULL_DOWN
#define PHDRIVER_PIN_IRQ_PULL_CFG      PH_DRIVER_PULL_UP
#define PHDRIVER_PIN_NSS_PULL_CFG      PH_DRIVER_PULL_UP

/******************************************************************
 * IRQ PIN NVIC settings
 ******************************************************************/
#define EINT_IRQn                 PORTB_PORTC_IRQn
#define EINT_PRIORITY             8
#define PIN_IRQ_TRIGGER_TYPE      PH_DRIVER_INTERRUPT_FALLINGEDGE

/*****************************************************************
 * Front End Reset logic level settings
 ****************************************************************/
#define PH_DRIVER_SET_HIGH            1          /**< Logic High. */
#define PH_DRIVER_SET_LOW             0          /**< Logic Low. */
#define RESET_POWERDOWN_LEVEL         PH_DRIVER_SET_HIGH
#define RESET_POWERUP_LEVEL           PH_DRIVER_SET_LOW


/*****************************************************************
 * I2C Configuration
 ****************************************************************/
#define PHDRIVER_KW41Z_I2C_POLLING                  /* Enable to perform I2C transfer using polling method. */
#define PHDRIVER_KW41Z_I2C_MASTER_BASEADDR          I2C1
#define PHDRIVER_KW41Z_I2C_BIT_RATE                 100000
#define PHDRIVER_KW41Z_I2C_MASTER_CLK_SRC           I2C1_CLK_SRC
#define PHDRIVER_KW41Z_I2C_MASTER_IRQ               I2C1_IRQn

#define I2C_IRQ_PRIORITY     7

#define PIN2_IDX 2
#define PIN3_IDX 3

#define ENABLE_PORT_SCL_I2C1  kCLOCK_PortC
#define PORT_SCL_I2C1         PORTC           /* Port C */
#define SCL_PINNUM_I2C1       38              /* Port C pin2 */

#define ENABLE_PORT_SDA_I2C1  kCLOCK_PortC
#define PORT_SDA_I2C1         PORTC           /* Port C */
#define SDA_PINNUM_I2C1       39              /* Port c pin 3 */



/*****************************************************************
 * Timer Configuration
 ****************************************************************/
#define PH_DRIVER_KW41Z_PIT_TIMER          PIT
#define PH_DRIVER_KW41Z_PIT_CLK            kCLOCK_BusClk
#define PH_DRIVER_KW41Z_TIMER_CHANNEL      kPIT_Chnl_0    /**< PIT channel number 0 */
#define PH_DRIVER_KW41Z_TIMER_NVIC         PIT_IRQn


#endif /* BOARD_FRDM_KW41Z_H_ */
