/*
 * phbalReg_LPC11U37_I2C.c
 *
 *  Created on: May 5, 2017
 *      Author: nxp79566
 */

#include "phDriver.h"
#include "phbalReg.h"
#include "BoardSelection.h"
#include "board.h"



/**
 * \brief Initialize the LPC11U37 I2C BAL layer.
 *
 * \return Status code
 * \retval #PH_DRIVER_SUCCESS Operation successful.
 * \retval #PH_ERR_INVALID_DATA_PARAMS Parameter structure size is invalid.
 */
phStatus_t phbalReg_Init(
        void * pDataParams,
        uint16_t wSizeOfDataParams
)
{
	uint8_t buffer[2];
    if((pDataParams == NULL) || (sizeof(phbalReg_Type_t) != wSizeOfDataParams))
    {
        return (PH_DRIVER_ERROR | PH_COMP_DRIVER);
    }

    ((phbalReg_Type_t *)pDataParams)->wId      = PH_COMP_DRIVER | PHBAL_REG_LPC11U37_I2C_ID;
    ((phbalReg_Type_t *)pDataParams)->bBalType = PHBAL_REG_TYPE_I2C;


    /* Configuring the I2C1 of LPC11u37 */
    Chip_SYSCTL_PeriphReset(RESET_I2C0);
	Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 4, IOCON_FUNC1 | I2C_FASTPLUS_BIT);
	Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 5, IOCON_FUNC1 | I2C_FASTPLUS_BIT);

	/* Initialize I2C */
	Chip_I2C_Init(I2C0);
	Chip_I2C_SetClockRate(I2C0, I2C_DEFAULT_SPEED);

	/* Set default mode to interrupt */
	Chip_I2C_SetMasterEventHandler(I2C0, Chip_I2C_EventHandler);
	NVIC_EnableIRQ(I2C0_IRQn);

	Chip_I2C_MasterRead(I2C0, PHDRIVER_I2CSLAVE_ADDRESS, buffer, I2C_MASTER_READ_LENGTH);

    return PH_DRIVER_SUCCESS;
}

phStatus_t phbalReg_Exchange(
        void * pDataParams,    /**< [In] Pointer to this layer's parameter structure. */
        uint16_t wOption,      /**< [In] Option parameter, for future use. */
        uint8_t * pTxBuffer,   /**< [In] Data to transmit. */
        uint16_t wTxLength,    /**< [In] Number of bytes to transmit, if 0 Tx is not performed. */
        uint16_t wRxBufSize,   /**< [In] Size of receive buffer / Number of bytes to receive (depending on implementation). If 0 Rx is not performed.  */
        uint8_t * pRxBuffer,   /**< [Out] Received data. */
        uint16_t * pRxLength   /**< [Out] Number of received data bytes. */
)
{
	static I2C_XFER_T xfer;
	uint8_t *pRxBuf;
	uint8_t g_dummyBuffer[260];

	 if(pDataParams == NULL)
	{
		return (PH_DRIVER_FAILURE | PH_COMP_DRIVER);
	}

	if(pRxBuffer == NULL)
	{
		pRxBuf = g_dummyBuffer;
	}
	else
	{
		pRxBuf = pRxBuffer;
	}

	/* If TxLength and Rxbuf is 0, return as failure */
	if (( wTxLength == 0 ) && ( wRxBufSize == 0 ))
	{
		return (PH_DRIVER_FAILURE | PH_COMP_DRIVER);
	}


	/* Setup I2C transfer record */
	xfer.slaveAddr = PHDRIVER_I2CSLAVE_ADDRESS;
	xfer.txBuff = pTxBuffer;
	xfer.txSz = wTxLength;
	xfer.rxBuff = pRxBuf;
	xfer.rxSz = wRxBufSize;
	xfer.status = 0;

	Chip_I2C_MasterTransfer(I2C0, &xfer);

	return (xfer.status != I2C_STATUS_DONE);

}

void I2C_IRQHandler(void)
{
	if (Chip_I2C_IsMasterActive(I2C0))
	{
		Chip_I2C_MasterStateHandler(I2C0);
	}
	else
	{
		Chip_I2C_SlaveStateHandler(I2C0);
	}
}

