/*
 * Copyright (c) 2016, NXP Semiconductor
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#ifndef NTAG_DEFINES_H
#define NTAG_DEFINES_H
/** @file ntag_defines.h
 * \brief Constants used by the NTAG I2C tag driver.
 */

#define NTAG_MAX_WRITE_DELAY_MS 10

/* Block Size for NTAG5 SRAM */
#define NTAG_I2C_BLOCK_SIZE 0x04

/* Layout of SRAM */
#define NTAG_MEM_BLOCK_START_SRAM  0x2000
#define NTAG_MEM_BLOCK_END_SRAM    0x203F
#define NTAG_MEM_SRAM_BLOCKS       0x3F
#define NTAG_MEM_SESSION_START     0x10A0
#define NTAG_MEM_SESSION_END       0x10AF
#define NTAG_MEM_EEPROM_END        0x01FF
#define NTAG_MEM_CONFIGURATION_START  0x1000

#ifndef NULL
#define NULL 0
#endif

/**
 * \brief Status that is expected during a NTAG Transaction
 */
typedef enum
{
    NTAG_OK,
    NTAG_ERROR_TX_FAILED,
    NTAG_ERROR_RX_FAILED,
    NTAG_ERROR_WRITE_TIMEOUT,
    NTAG_ERROR_INVALID_PARAM,
    NTAG_CLOSED,
    NTAG_EEPROM_WRITE_ERROR,
    NTAG_STATUS_MAX_NUMBER
} NTAG_STATUS_T;

//----------------------------------------------------------------------
///
///  byte offset in session memory
#define NTAG_MEM_OFFSET_TAG_STATUS_REG            0x00
#define NTAG_MEM_OFFSET_TAG_CONFIG0_REG           0x01
#define NTAG_MEM_OFFSET_TAG_CONFIG1_REG           0x02
#define NTAG_MEM_OFFSET_NDEF_DATA_BLOCK_REG       0x03
#define NTAG_MEM_OFFSET_PWM_GPIO_CONFIG_REG       0x04
#define NTAG_MEM_OFFSET_PWM0_ON_REG               0x05
#define NTAG_MEM_OFFSET_PWM0_OFF_REG              0x06
#define NTAG_MEM_OFFSET_PWM1_ON_REG               0x07
#define NTAG_MEM_OFFSET_PWM1_OFF_REG              0x08
#define NTAG_MEM_OFFSET_WDT_CONF_REG              0x09
#define NTAG_MEM_OFFSET_WDT_EN_REG                0x0A
#define NTAG_MEM_OFFSET_EH_CONF_REG               0x0B
#define NTAG_MEM_OFFSET_ED_CONF_REG               0x0C
#define NTAG_MEM_OFFSET_I2C_SLAVE_CONF_REG        0x0D
#define NTAG_MEM_OFFSET_RESET_GEN_REG             0x0E
#define NTAG_MEM_OFFSET_ED_INTR_CLEAR_REG         0x0F
#define NTAG_MEM_OFFSET_I2CM_SLAVE_ADDR_REG       0x10
#define NTAG_MEM_OFFSET_I2CM_DATA_LEN_REG         0x11
#define NTAG_MEM_OFFSET_ALM_STATUS_REG            0x12
#define NTAG_MEM_OFFSET_I2CM_STATUS_REG           0x13

//----------------------------------------------------------------------
///
/// error codes

#define NTAG_ERR_OK 0x00
#define NTAG_ERR_COMMUNICATION -0x01
#define NTAG_ERR_BUFF_OVERFLOW -0x02
#define NTAG_ERR_INIT_FAILED -0x03
#define NTAG_ERR_INVALID_PARAM -0x09

#endif // NTAG_DEFINES_H
