/*
 * Copyright (c) 2016, NXP Semiconductor
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#ifndef _NTAG_DRIVER_INTERN_H_
#define _NTAG_DRIVER_INTERN_H_

/***********************************************************************/
/* INCLUDES                                                            */
/***********************************************************************/
#include "ntag_driver.h"
#ifdef HAVE_NTAG_INTERRUPT
#include "HAL_ISR_driver.h"
#endif
/***********************************************************************/
/* DEFINES                                                             */
/***********************************************************************/

#define I2C_KCH_ADDRESS                           0x1030         /* Key Configuration Header */
#define I2C_PP_ADDRESS                            0x1031         /* Protection Pointer */
#define I2C_PPC_ADDRESS                           0x1031         /* Protection Pointer Configuration */
#define I2C_RESTRICTED_AREA_ADDRESS               0x103F         /* Restricted area address */
#define I2C_NEGATIVE_AUTH_COUNTER                 0x1032         /* Negative Auth Counter address */
#define I2C_READ_KEY_ADDRESS                      0x1033         /* Key address for read operation */
#define I2C_WRITE_KEY_ADDRESS                     0x1034         /* Key address for write operation */
#define I2C_RESTRICTED_READ_KEY_ADDRESS           0x1035         /* Key address for Restricted Read key operation */
#define I2C_RESTRICTED_WRITE_KEY_ADDRESS          0x1036         /* Key address for Restricted write key operation */
#define I2C_READ_AUTHENTICATE                     0x1096         /* Address to authenticate Read */
#define I2C_WRITE_AUTHENTCATE                     0x1097         /* Address to authenticate Write */
#define I2C_RESTRICTED_READ_AUTHENTICATE          0x1098         /* Address to authenticate Restricted Read */
#define I2C_RESTRICTED_WRITE_AUTHENTICATE         0x1099         /* Address to authenticate Restricted Write */

#define NTAG_RF_SECTION_LOCK0_ADDRESS             0x1092         /* Address to Lock Sections from RF interface */
#define NTAG_RF_SECTION_LOCK1_ADDRESS             0x1093         /* Address to Lock Sections from RF interface */
#define NTAG_I2C_SECTION_LOCK0_ADDRESS            0x1094         /* Address to Lock Sections from I2C interface */
#define NTAG_I2C_SECTION_LOCK1_ADDRESS            0x1095         /* Address to Lock Sections from I2C interface */
#define NTAG_I2C_WDT_CONF_ADDRESS                 0x103C         /* WDT Timer Configuration */

#define NTAG_RF_LOCK_BLOCK_OFFSET                 0x106A         /* Address to Lock Block from RF interface */
#define NTAG_I2C_LOCK_BLOCK_OFFSET                0x108A         /* Address to Lock Block from I2C interface */

#define NTAG_MEM_I2C_NFC_SILENCE_MASK             0x0020         /* Mask for NFC Silence */
#define NTAG_MEM_I2C_NFC_SILENCE_REVERSE          0xFFDF         /* Reverse Mask for NFC Silence */
#define NTAG_MEM_I2C_IF_LOCKED_MASK               0x0200         /* Mask for I2C IF Locked */
#define NTAG_MEM_PT_TRANSFER_DIR_MASK             0x0100
#define NTAG_MEM_PT_TRANSFER_DIR_REVERSE          0xFEFF
#define NTAG_MEM_AUTO_STANDBY_EN_MASK             0x0001
#define NTAG_MEM_ED_FUNCTION_MASK                 0x000F
#define NTAG_MEM_CLEAR_INTR_MASK                  0x0001         /* Mask for Clearing ED Intr */

//----------------------------------------------------------------------
///
///  Operation for I2C

#define NTAG_MEM_TAG_STATUS_MASK_EEPROM_WR_BUSY       0x0080
#define NTAG_MEM_TAG_STATUS_MASK_EEPROM_WR_ERR        0x0040
#define NTAG_MEM_SRAM_MIRROR_MASK                     0x0400
#define NTAG_MEM_SRAM_MIRROR_REVERSE_MASK             0xFBFF
#define NTAG_MEM_SRAM_PT_MASK                         0x0800
#define NTAG_MEM_SRAM_PT_REVERSE_MASK                 0xF7FF
#define NTAG_MEM_SRAM_PHDC_MASK                       0x0C00
#define NTAG_MEM_SRAM_PHDC_REVERSE_MASK               0xF3FF
#define NTAG_MEM_I2C_SOFTRESET_MASK                   0x1000
#define NTAG_MEM_I2C_SOFTRESET_REVERSE                0xEFFF
/***********************************************************************/
/* TYPES                                                               */
/***********************************************************************/
/**
 * \brief Session Register with their respective address.
 */
static const uint16_t SessionRegister[][3] =
{
        /* Block Address, Byte Offset, Byte Width */
        {0x10A0,          0,              2},     /* Tag Status Reg */
        {0x10A1,          0,              2},     /* Tag Config Reg 0 */
        {0x10A1,          2,              2},     /* Tag Config Reg 1 */
        {0x10A2,          0,              2},     /* Ndef Data Block Reg */
        {0x10A3,          0,              2},     /* PWM GPIO Config Reg */
        {0x10A4,          0,              2},     /* PWM0 ON Reg */
        {0x10A4,          2,              2},     /* PWM0 OFF Reg */
        {0x10A5,          0,              2},     /* PWM1 ON Reg */
        {0x10A5,          2,              2},     /* PWM1 OFF Reg */
        {0x10A6,          0,              2},     /* WDT CONF Reg */
        {0x10A6,          2,              1},     /* WDT EN Reg */
        {0x10A7,          0,              1},     /* EH CONF Reg */
        {0x10A8,          0,              1},     /* ED CONF Reg */
        {0x10A9,          0,              2},     /* I2C SLAVE CONF Reg */
        {0x10AA,          0,              1},     /* RESET GEN Reg */
        {0x10AB,          0,              1},     /* ED INTR CLEAR Reg */
        {0x10AC,          0,              1},     /* I2CM SLAVE ADDR Reg */
        {0x10AC,          1,              1},     /* I2CM DATA LEN Reg */
        {0x10AC,          2,              1},     /* ALM Status Reg */
        {0x10AD,          0,              1}      /* I2CM STATUS Reg */
};

/***********************************************************************/
/* GLOBAL VARIABLES                                                    */
/***********************************************************************/

/***********************************************************************/
/* GLOBAL FUNCTION PROTOTYPES                                          */
/***********************************************************************/
/**
 * \brief read the specified block from the selected NTAG device
 *
 *	This functions reads the specified 4-byte sized block from the selected
 *	NTAG device. Reading an invalid block will result in a failed read.
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	bytes	array of bytes to store read data
 * \param	block	block number to read
 * \param	len 	number of bytes to be read
 *
 * \return			true on failure
 */
BOOL NTAG_ReadBlock(NTAG_HANDLE_T ntag, uint16_t block, uint8_t *bytes, uint8_t len);

/**
 * \brief write the specified block to the selected NTAG device
 *
 *	This functions writes the specified 4-byte sized block to selected NTAG device.
 *	Writing to an invalid block is undefined. Writing less than a full block will write
 *	0 to the rest of the block.
 *
 *	When directly using this function care has to be taken when writing the first byte
 *	of the first block( block = 0 ) because it will modify the I2C address of the device
 *	whereas on read the first byte will contain part of the serial number.
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	bytes	array of bytes to be written
 * \param	block	block number to write
 * \param	len 	number of bytes to be written
 *
 * \return			true on failure
 */
BOOL NTAG_WriteBlock(NTAG_HANDLE_T ntag, uint16_t block, const uint8_t *bytes, uint8_t len);

#endif /* _NTAG_DRIVER_INTERN_H_ */
