/*
 * Copyright (c) 2016, NXP Semiconductor
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/***********************************************************************/
/* INCLUDES                                                            */
/***********************************************************************/

#include "ntag_driver_intern.h"
#include "ntag_defines.h"
#include "global_types.h"
#include "phDriver.h"
#include "phDriver_Timer.h"
#include "ntag_auth.h"
/***********************************************************************/
/* DEFINES                                                             */
/***********************************************************************/
extern void phDriver_PitTimerCallBack(void);
/***********************************************************************/
/* GLOBAL VARIABLES                                                    */
/***********************************************************************/
extern uint8_t Unblock;
/***********************************************************************/
/* GLOBAL PUBLIC FUNCTIONS                                             */
/***********************************************************************/

BOOL NTAG_I2C_WRITE_KEY(NTAG_HANDLE_T ntag, NTAG_OPERATION_T operation, uint8_t* byte)
{
    uint16_t address = 0;
    BOOL status;

    if((ntag == NULL) || (byte == NULL))
    {
        return true;
    }

    /* Address of configuration memory to be picked for the operation */
    switch(operation)
    {
    case NTAG_READ:
        address = I2C_READ_KEY_ADDRESS;
        break;
    case NTAG_WRITE:
        address = I2C_WRITE_KEY_ADDRESS;
        break;
    case NTAG_RESTRICTEDREAD:
        address = I2C_RESTRICTED_READ_KEY_ADDRESS;
        break;
    case NTAG_RESTRICTEDWRITE:
        address = I2C_RESTRICTED_WRITE_KEY_ADDRESS;
        break;
    default:
        return true;
        break;
    }

    /* Write the key to the respective address */
    status = NTAG_WriteBlock(ntag, address, byte, NTAG_I2C_BLOCK_SIZE);
    return status;
}

BOOL NTAG_I2C_READ_KEY(NTAG_HANDLE_T ntag, NTAG_OPERATION_T operation, uint8_t* byte)
{
    uint16_t address = 0;
    BOOL status;

    if((ntag == NULL) || (byte == NULL))
    {
        return true;
    }

    /* Address of configuration memory to be picked for the operation */
    switch(operation)
    {
    case NTAG_READ:
        address = I2C_READ_KEY_ADDRESS;
        break;
    case NTAG_WRITE:
        address = I2C_WRITE_KEY_ADDRESS;
        break;
    case NTAG_RESTRICTEDREAD:
        address = I2C_RESTRICTED_READ_KEY_ADDRESS;
        break;
    case NTAG_RESTRICTEDWRITE:
        address = I2C_RESTRICTED_WRITE_KEY_ADDRESS;
        break;
    default:
        return true;
        break;
    }

    /* Read the key from the respective address */
    status = NTAG_ReadBlock(ntag, address, byte, NTAG_I2C_BLOCK_SIZE);
    return status;
}


BOOL NTAG_I2C_AUTHENTICATION(NTAG_HANDLE_T ntag, NTAG_OPERATION_T operation, uint8_t* byte)
{
    uint16_t address = 0;
    BOOL status;

    if((ntag == NULL) || (byte == NULL))
    {
        return true;
    }

    /* Address of configuration memory to be picked for the operation */
    switch(operation)
    {
    case NTAG_READ:
        address = I2C_READ_AUTHENTICATE;
        break;
    case NTAG_WRITE:
        address = I2C_WRITE_AUTHENTCATE;
        break;
    case NTAG_RESTRICTEDREAD:
        address = I2C_RESTRICTED_READ_AUTHENTICATE;
        break;
    case NTAG_RESTRICTEDWRITE:
        address = I2C_RESTRICTED_WRITE_AUTHENTICATE;
        break;
    default:
        return true;
        break;
    }

    /* Write the key to the respective address to authenticate the block*/
    status = NTAG_WriteBlock(ntag, address, byte, NTAG_I2C_BLOCK_SIZE);

    return status;
}


BOOL NTAG_I2C_SET_SecurityParameter(NTAG_HANDLE_T ntag, NTAG_SECURITY_PARAMETER_T Parameter, uint16_t value)
{
    uint16_t address = 0;
    BOOL status;
    uint8_t data[4] = {0x00};

    if(ntag == NULL)
    {
        return true;
    }

    /* Address of configuration memory to be picked for the operation */
    switch(Parameter)
    {
    case NTAG_KCH:
        address = I2C_KCH_ADDRESS;
        status = NTAG_ReadBlock(ntag, address, &data[0], NTAG_I2C_BLOCK_SIZE);
        data[0] = (uint8_t)value;
        break;
    case NTAG_PROTECTIONPOINTER:
        address = I2C_PP_ADDRESS;
        status = NTAG_ReadBlock(ntag, address, &data[0], NTAG_I2C_BLOCK_SIZE);
        data[0] = (uint8_t)value;
        break;
    case NTAG_PROTECTIONPOINTER_CONFIGURATION:
        address = I2C_PPC_ADDRESS;
        status = NTAG_ReadBlock(ntag, address, &data[0], NTAG_I2C_BLOCK_SIZE);
        data[1] = (uint8_t)value;
        break;
    case NTAG_RESTRICTEDAREA:
        address = I2C_RESTRICTED_AREA_ADDRESS;
        status = NTAG_ReadBlock(ntag, address, &data[0], NTAG_I2C_BLOCK_SIZE);
        data[2] = (uint8_t)value;
        data[3] = (uint8_t)(value >>8);

        break;
    default:
        return true;
        break;
    }

    /* Write the value to the respective block */
    status = NTAG_WriteBlock(ntag, address, &data[0], NTAG_I2C_BLOCK_SIZE);
    return status;

}

BOOL NTAG_I2C_GET_SecurityParameter(NTAG_HANDLE_T ntag, NTAG_SECURITY_PARAMETER_T Parameter, uint16_t* value)
{
    uint16_t address = 0;
    BOOL status;
    uint8_t data[4] = {0x00};
    if((ntag == NULL) || (value == NULL))
    {
        return true;
    }

    /* Address of configuration memory to be picked for the operation */
    switch(Parameter)
    {
    case NTAG_KCH:
        address = I2C_KCH_ADDRESS;
        status = NTAG_ReadBlock(ntag, address, &data[0], NTAG_I2C_BLOCK_SIZE);
        *value = data[0];
        break;
    case NTAG_PROTECTIONPOINTER:
        address = I2C_PP_ADDRESS;
        status = NTAG_ReadBlock(ntag, address, &data[0], NTAG_I2C_BLOCK_SIZE);
        *value = data[0];
        break;
    case NTAG_PROTECTIONPOINTER_CONFIGURATION:
        address = I2C_PPC_ADDRESS;
        status = NTAG_ReadBlock(ntag, address, &data[0], NTAG_I2C_BLOCK_SIZE);
        *value = data[1];
        break;
    case NTAG_RESTRICTEDAREA:
        address = I2C_RESTRICTED_AREA_ADDRESS;
        status = NTAG_ReadBlock(ntag, address, &data[0], NTAG_I2C_BLOCK_SIZE);
        *value = (uint16_t)(((uint16_t)(data[3] << 8)) | data[2]);
        break;
    default:
        return true;
        break;
    }

    return status;
}

BOOL NTAG_I2C_Set_AuthLimit(NTAG_HANDLE_T ntag, uint16_t value)
{
    uint8_t data[4] = {0};
    BOOL status;
    if(ntag == NULL)
    {
        return true;
    }
    /* Reading out the entire block so as not to over ride the previous value */
    status = NTAG_ReadBlock(ntag, I2C_NEGATIVE_AUTH_COUNTER, &data[0], NTAG_I2C_BLOCK_SIZE);
    if(status == true)
    {
        return status;
    }
    data[0] = (uint8_t)(value); /* LSB */
    data[1] &= 0x80; /* Clearing the bits other than MSB */
    data[1] |= ((uint8_t)(value >> 8) & 0x7F); /* MSB */
    status = NTAG_WriteBlock(ntag, I2C_NEGATIVE_AUTH_COUNTER, &data[0], NTAG_I2C_BLOCK_SIZE);
    return status;

}


BOOL NTAG_I2C_Read_AuthLimit(NTAG_HANDLE_T ntag, uint16_t* value)
{
    uint8_t data[4] = {0};
    BOOL status;
    if((ntag == NULL) || (value == NULL))
    {
        return true;
    }
    /* Reading the auth limit counter block to read out the couter value */
    status = NTAG_ReadBlock(ntag, I2C_NEGATIVE_AUTH_COUNTER, &data[0], NTAG_I2C_BLOCK_SIZE);
    *value = (uint16_t)(((uint16_t)(data[1] << 8)) | data[0]);
    return status;
}

BOOL NTAG_I2C_AuthCounter_OnOff(NTAG_HANDLE_T ntag, Bool on)
{
    uint8_t data[4] = {0};
    BOOL status;
    if(ntag == NULL)
    {
        return true;
    }
    /* Reading out the block so as not to override previous value */
    status = NTAG_ReadBlock(ntag, I2C_NEGATIVE_AUTH_COUNTER, &data[0], NTAG_I2C_BLOCK_SIZE);
    if(status == true)
    {
        return status;
    }
    if(on)
    {
        data[1] |= 0x80;
    }
    else
    {
        data[1] &= 0x7F;
    }
    /* Writing the block so as either to disable or enable the counter */
    status = NTAG_WriteBlock(ntag, I2C_NEGATIVE_AUTH_COUNTER, &data[0], NTAG_I2C_BLOCK_SIZE);
    return status;
}

