/*
 * Copyright (c) 2016, NXP Semiconductor
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/***********************************************************************/
/* INCLUDES                                                            */
/***********************************************************************/

#include "ntag_driver_intern.h"
#include "ntag_defines.h"
#include "global_types.h"
#include "phDriver.h"
#include "phDriver_Timer.h"
#include "ntag_bridge.h"

/***********************************************************************/
/* DEFINES                                                             */
/***********************************************************************/
#define CLEAR_ED_INTR                0x0001
/***********************************************************************/
/* GLOBAL VARIABLES                                                    */
/***********************************************************************/
extern uint8_t Unblock;
/***********************************************************************/
/* GLOBAL PUBLIC FUNCTIONS                                             */
/***********************************************************************/

BOOL NTAG_WaitForEvent(NTAG_HANDLE_T ntag, NTAG_EVENT_T event, uint32_t timeout_ms, BOOL set_fd_pin_function)
{
    if(ntag == NULL)
    {
        return true;
    }

    /* Set the Function if asked to */
    if(set_fd_pin_function)
    {
        switch(event)
        {
        case NTAG_EVENT_ED_PIN_HI_INTERRUPT:
            break;

        case NTAG_EVENT_ED_PIN_LO_INTERRUPT:
            break;

        case NTAG_EVENT_RF_FIELD_PRESENT_INTERRUPT:
            NTAG_SetEDFunction(ntag, RF_SWITCHED_ON_OR_OFF_0001b);
            break;

        case NTAG_EVENT_RF_FIELD_NOT_PRESENT_INTERRUPT:
            NTAG_SetEDFunction(ntag, RF_SWITCHED_ON_OR_OFF_0001b);
            break;

        case NTAG_EVENT_PWM0_ON:
            NTAG_SetEDFunction(ntag, PWM0_Signal_0010b);
            break;

        case NTAG_EVENT_PWM0_OFF:
            NTAG_SetEDFunction(ntag, PWM0_Signal_0010b);
            break;

        case NTAG_EVENT_LAST_BYTE_READ_FROM_RF:
            NTAG_SetEDFunction(ntag, PASS_THROUGH_MODE_I2C_TO_RF_0011b);
            break;

        case NTAG_EVENT_LAST_BYTE_WRITTEN_BY_I2C:
            NTAG_SetEDFunction(ntag, PASS_THROUGH_MODE_I2C_TO_RF_0011b);
            break;

        case NTAG_EVENT_LAST_BYTE_WRITTEN_BY_RF:
            NTAG_SetEDFunction(ntag, PASS_THROUGH_MODE_RF_TO_I2C_0100b);
            break;

        case NTAG_EVENT_LAST_BYTE_READ_FROM_I2C:
            NTAG_SetEDFunction(ntag, PASS_THROUGH_MODE_RF_TO_I2C_0100b);
            break;

        case NTAG_EVENT_ARBITER_LOCKED_BY_RF:
            NTAG_SetEDFunction(ntag, ARBITER_LOCKED_TO_RF_0101b);
            break;
        case NTAG_EVENT_ARBITER_NOT_LOCKED_BY_RF:
            NTAG_SetEDFunction(ntag, ARBITER_LOCKED_TO_RF_0101b);
            break;
        case NTAG_EVENT_NDEF_LENGTH_BYTE_NONZERO:
            NTAG_SetEDFunction(ntag, NDEFDATA_ZERO_OR_NONZERO_0110b);
            break;
        case NTAG_EVENT_NDEF_LENGTH_BYTE_ZERO:
            NTAG_SetEDFunction(ntag, NDEFDATA_ZERO_OR_NONZERO_0110b);
            break;
        case NTAG_EVENT_STANDBY_INACTIVE:
            NTAG_SetEDFunction(ntag, STANDBY_MODE_0111b);
            break;
        case NTAG_EVENT_STANDBY_ACTIVE:
            NTAG_SetEDFunction(ntag, STANDBY_MODE_0111b);
            break;
        case NTAG_EVENT_START_OF_WRITE:
            NTAG_SetEDFunction(ntag, WRITE_COMMAND_ONGOING_1000b);
            break;
        case NTAG_EVENT_END_OF_WRITE:
            NTAG_SetEDFunction(ntag, WRITE_COMMAND_ONGOING_1000b);
            break;
        case NTAG_EVENT_START_OF_READ:
            NTAG_SetEDFunction(ntag, READ_COMMAND_ONGOING_1001b);
            break;
        case NTAG_EVENT_END_OF_READ:
            NTAG_SetEDFunction(ntag, READ_COMMAND_ONGOING_1001b);
            break;
        case NTAG_EVENT_SOF_DETECTED:
            NTAG_SetEDFunction(ntag, SOF_OR_EOF_DETECTED_1010b);
            break;
        case NTAG_EVENT_EOF_DETECTED:
            NTAG_SetEDFunction(ntag, SOF_OR_EOF_DETECTED_1010b);
            break;
        case NTAG_EVENT_NDEF_DATA_BLOCK_READ:
            NTAG_SetEDFunction(ntag, READ_FROM_NDEF_DATA_BLOCK_OR_FIELD_OFF_1011b);
            break;
        case NTAG_EVENT_NDEF_DATA_BLOCK_WRITTEN:
            NTAG_SetEDFunction(ntag, WRITTEN_TO_NDEF_DATA_BLOCK_OR_FIELD_OFF_1100b);
            break;
        case NTAG_EVENT_SOFTWARE_INTERRUPT:
            NTAG_SetEDFunction(ntag, SOFTWARE_INITIATED_INTERRUPT_1101b);
            break;
        case NTAG_EVENT_ACTIVE_RF_DETECT_ON:
            NTAG_SetEDFunction(ntag, ACTIVE_RF_DETECT_1110b);
            break;
        case NTAG_EVENT_ACTIVE_RF_DETECT_OFF:
            NTAG_SetEDFunction(ntag, ACTIVE_RF_DETECT_1110b);
            break;
        case NTAG_EVENT_ACTIVE_RF_OK:
            NTAG_SetEDFunction(ntag, ACTIVE_RF_OK_1111b);
            break;
        case NTAG_EVENT_ACTIVE_RF_NOT_OK:
            NTAG_SetEDFunction(ntag, ACTIVE_RF_OK_1111b);
            break;
        default:
            break;
        }
    }

    /* Wait on the event depending upon whether HI or LOW interrupt is expected */
    switch(event)
    {

    case NTAG_EVENT_ED_PIN_LO_INTERRUPT:
    case NTAG_EVENT_RF_FIELD_PRESENT_INTERRUPT:
    case NTAG_EVENT_PWM0_OFF:
    case NTAG_EVENT_LAST_BYTE_READ_FROM_RF:
    case NTAG_EVENT_LAST_BYTE_WRITTEN_BY_RF:
    case NTAG_EVENT_ARBITER_LOCKED_BY_RF:
    case NTAG_EVENT_NDEF_LENGTH_BYTE_NONZERO:
    case NTAG_EVENT_STANDBY_INACTIVE:
    case NTAG_EVENT_START_OF_WRITE:
    case NTAG_EVENT_START_OF_READ:
    case NTAG_EVENT_SOF_DETECTED:
    case NTAG_EVENT_NDEF_DATA_BLOCK_READ:
    case NTAG_EVENT_NDEF_DATA_BLOCK_WRITTEN:
    case NTAG_EVENT_SOFTWARE_INTERRUPT:
    case NTAG_EVENT_ACTIVE_RF_DETECT_ON:
    case NTAG_EVENT_ACTIVE_RF_OK:
        return NTAG_WaitForEDPinState(ntag, NTAG_ED_PIN_STATE_LO, timeout_ms);
        break;

    case NTAG_EVENT_ED_PIN_HI_INTERRUPT:
    case NTAG_EVENT_RF_FIELD_NOT_PRESENT_INTERRUPT:
    case NTAG_EVENT_PWM0_ON:
    case NTAG_EVENT_LAST_BYTE_WRITTEN_BY_I2C:
    case NTAG_EVENT_LAST_BYTE_READ_FROM_I2C:
    case NTAG_EVENT_ARBITER_NOT_LOCKED_BY_RF:
    case NTAG_EVENT_NDEF_LENGTH_BYTE_ZERO:
    case NTAG_EVENT_STANDBY_ACTIVE:
    case NTAG_EVENT_END_OF_WRITE:
    case NTAG_EVENT_END_OF_READ:
    case NTAG_EVENT_EOF_DETECTED:
    case NTAG_EVENT_ACTIVE_RF_DETECT_OFF:
    case NTAG_EVENT_ACTIVE_RF_NOT_OK:
        return NTAG_WaitForEDPinState(ntag, NTAG_ED_PIN_STATE_HI, timeout_ms);
        break;
    default:
        break;
    }
    return 1;
}

BOOL NTAG_ClearED(NTAG_HANDLE_T ntag)
{
    BOOL status;
    if(ntag == NULL)
    {
        return true;
    }

    /* Clear the ED pin, pulls the ED Pin High  */
    status = NTAG_WriteRegister(ntag, NTAG_MEM_OFFSET_ED_INTR_CLEAR_REG, NTAG_MEM_CLEAR_INTR_MASK, CLEAR_ED_INTR);
    return status;
}

BOOL NTAG_SetI2CRstOnOff(NTAG_HANDLE_T ntag, BOOL on)
{
    BOOL status;
    uint16_t value = 0;
    if(ntag == NULL)
    {
        return true;
    }

    if(on)
    {
        value = NTAG_MEM_I2C_SOFTRESET_MASK;
    }
    else
    {
        value = NTAG_MEM_I2C_SOFTRESET_REVERSE;
    }
    /* Setting the I2C reset */
    status = NTAG_WriteRegister(ntag, NTAG_MEM_OFFSET_I2C_SLAVE_CONF_REG, NTAG_MEM_I2C_SOFTRESET_MASK, value);
    return status;
}


BOOL NTAG_SetI2C_SectionLock(NTAG_HANDLE_T ntag, NTAG_SECTION_LOCK_T SectionLock, uint8_t value)
{
    BOOL status;
    uint8_t data[4] = {0};
    uint16_t address;
    data[0] = value;

    if(ntag == NULL)
    {
        return true;
    }

    /* Select the address for the section Lock */
    switch(SectionLock)
    {
        case RF_SECTION_LOCK0:
            address = NTAG_RF_SECTION_LOCK0_ADDRESS;
        break;
        case RF_SECTION_LOCK1:
            address = NTAG_RF_SECTION_LOCK1_ADDRESS;
        break;
        case I2C_SECTION_LOCK0:
            address = NTAG_I2C_SECTION_LOCK0_ADDRESS;
        break;
        case I2C_SECTION_LOCK1:
            address = NTAG_I2C_SECTION_LOCK1_ADDRESS;
        break;
        default:
            return true;
        break;
    }
    /* Write the section lock configuration memory with the data provided */
    status =  NTAG_WriteBlock( ntag, address, &data[0],  NTAG_I2C_BLOCK_SIZE);
    return status;
}


BOOL NTAG_GetI2C_SectionLock(NTAG_HANDLE_T ntag, NTAG_SECTION_LOCK_T SectionLock, uint8_t* value)
{
    BOOL status;
    uint8_t data[4] = {0};
    uint16_t address;

    if(ntag == NULL)
    {
        return true;
    }

    /* Select the address for the section Lock */
    switch(SectionLock)
    {
        case RF_SECTION_LOCK0:
            address = NTAG_RF_SECTION_LOCK0_ADDRESS;
        break;
        case RF_SECTION_LOCK1:
            address = NTAG_RF_SECTION_LOCK1_ADDRESS;
        break;
        case I2C_SECTION_LOCK0:
            address = NTAG_I2C_SECTION_LOCK0_ADDRESS;
        break;
        case I2C_SECTION_LOCK1:
            address = NTAG_I2C_SECTION_LOCK1_ADDRESS;
        break;
        default:
            return true;
        break;
    }
    /* Read the section lock configuration memory */
    status =  NTAG_ReadBlock( ntag, address, &data[0],  NTAG_I2C_BLOCK_SIZE);
    *value = data[0];
    return status;
}


BOOL NTAG_SetI2C_SectionConfiguration(NTAG_HANDLE_T ntag, NTAG_SECTION_T Section, NTAG_LOCK_T Lock)
{
    uint16_t address;
    BOOL status;
    uint8_t data[4] = {0x00};
    if(ntag == NULL)
    {
        return true;
    }
    /* Select the configuration memory address for the lock based on interface and read/write */
    switch(Lock)
    {
        case RF_NOT_READABLE:
            address = NTAG_RF_SECTION_LOCK1_ADDRESS;
            break;
        case RF_NOT_WRITABLE:
            address = NTAG_RF_SECTION_LOCK0_ADDRESS;
            break;
        case I2C_NOT_READABLE:
            address = NTAG_I2C_SECTION_LOCK1_ADDRESS;
            break;
        case I2C_NOT_WRITABLE:
            address = NTAG_I2C_SECTION_LOCK0_ADDRESS;
            break;
        default:
            return true;
            break;
    }
    /* Read the block out so as not to interfere with other settings */
    status =  NTAG_ReadBlock( ntag, address, &data[0],  NTAG_I2C_BLOCK_SIZE);

    /* Set the corresponding bit of the first byte of block */
    switch(Section)
    {
        case NTAG_SECTION_0:
            data[0] |= 0x01;
            break;
        case NTAG_SECTION_1:
            data[0] |= 0x02;
            break;
        case NTAG_SECTION_2:
            data[0] |= 0x04;
            break;
        case NTAG_SECTION_3:
            data[0] |= 0x08;
            break;
        case NTAG_SECTION_4:
            data[0] |= 0x10;
            break;
        case NTAG_SECTION_5:
            data[0] |= 0x20;
            break;
        case NTAG_SECTION_6:
            data[0] |= 0x40;
            break;
        case NTAG_SECTION_7:
            data[0] |= 0x80;
            break;
        default:
            return true;
            break;
    }
    /* Write the selected configuration memory address */
    status =  NTAG_WriteBlock( ntag, address, &data[0],  NTAG_I2C_BLOCK_SIZE);
    return status;
}


BOOL NTAG_GetI2C_SectionConfiguration(NTAG_HANDLE_T ntag, NTAG_SECTION_T Section, NTAG_ACCESS_T Access, BOOL* on)
{
    uint16_t address;
    BOOL status;
    uint8_t data[4] = {0x00};
    if(ntag == NULL)
    {
        return true;
    }
    /* Select the configuration memory based on the access */
    switch(Access)
    {
        case RF_READABLE:
            address = NTAG_RF_SECTION_LOCK1_ADDRESS;
            break;
        case RF_WRITABLE:
            address = NTAG_RF_SECTION_LOCK0_ADDRESS;
            break;
        case I2C_READABLE:
            address = NTAG_I2C_SECTION_LOCK1_ADDRESS;
            break;
        case I2C_WRITABLE:
            address = NTAG_I2C_SECTION_LOCK0_ADDRESS;
            break;
        default:
            return true;
            break;
    }
    /* Read the corresponding configuration memory */
    status =  NTAG_ReadBlock( ntag, address, &data[0],  NTAG_I2C_BLOCK_SIZE);

    /* Parse the received data to check section against access condition */
    switch(Section)
    {
        case NTAG_SECTION_0:
            *on = !(data[0] & 0x01);
            break;
        case NTAG_SECTION_1:
            *on = !((data[0] & 0x02) >> 1);
            break;
        case NTAG_SECTION_2:
            *on = !((data[0] & 0x04) >> 2);
            break;
        case NTAG_SECTION_3:
            *on = !((data[0] & 0x08) >> 3);
            break;
        case NTAG_SECTION_4:
            *on = !((data[0] & 0x10) >> 4);
            break;
        case NTAG_SECTION_5:
            *on = !((data[0] & 0x20) >> 5);
            break;
        case NTAG_SECTION_6:
            *on = !((data[0] & 0x40) >> 6);
            break;
        case NTAG_SECTION_7:
            *on = !((data[0] & 0x80) >> 7);
            break;
        default:
            return true;
            break;
    }

    return status;

}


BOOL NTAG_SetI2C_LockBlock(NTAG_HANDLE_T ntag, NTAG_INTERFACE_T Interface, uint16_t BlockNo)
{
    BOOL status;
    uint8_t data[4] = {0x00};
    uint16_t address = (uint16_t)NTAG_RF_LOCK_BLOCK_OFFSET;
    uint16_t BlockIncrement = 0;
    uint16_t BitIncrement = 1;
    if(ntag == NULL)
    {
        return true;
    }

    switch(Interface)
    {
        case NTAG_RF_LOCK:
            /* The increment to be counted from the base address */
            BlockIncrement = BlockNo / 16;
            /* Increment the address according to offset */
            address = address + BlockIncrement;
            /* The bit order within the block */
            BitIncrement = (uint16_t)(BitIncrement << (BlockNo % 16));
            /* Reading out the already present data */
            status =  NTAG_ReadBlock( ntag, address, &data[0],  NTAG_I2C_BLOCK_SIZE);
            data[0] |=  (uint8_t)BitIncrement;
            data[1] |=  (uint8_t)(BitIncrement >> 8);
            status =  NTAG_WriteBlock( ntag, address, &data[0],  NTAG_I2C_BLOCK_SIZE);
        break;

        case NTAG_I2C_LOCK:
            /* Making the default address to be I2C address */
            address = (uint16_t)NTAG_I2C_LOCK_BLOCK_OFFSET;
            /* Per Configuration Block 64 blocks can be locked */
            BlockIncrement = BlockNo / 64;
            /* Incrementing the value of configuration block */
            address = address + BlockIncrement;
            /* The bit order within the block */
            BitIncrement = (uint16_t)(BitIncrement << ((BlockNo % 64) / 4));
            status =  NTAG_ReadBlock( ntag, address, &data[0],  NTAG_I2C_BLOCK_SIZE);
            data[0] |=  (uint8_t)BitIncrement;
            data[1] |=  (uint8_t)(BitIncrement >> 8);
            status =  NTAG_WriteBlock( ntag, address, &data[0],  NTAG_I2C_BLOCK_SIZE);
        break;

        default:
            return true;
            break;
    }
    return status;
}


BOOL NTAG_GetI2C_LockBlock(NTAG_HANDLE_T ntag, NTAG_INTERFACE_T Interface, uint16_t BlockNo, Bool* Locked)
{
    BOOL status;
    uint8_t data[4] = {0x00};
    uint16_t address = (uint16_t)NTAG_RF_LOCK_BLOCK_OFFSET;
    uint16_t BlockIncrement = 0;
    uint16_t BitIncrement = 1;
    uint8_t Lock;
    if(ntag == NULL)
    {
        return true;
    }
    switch(Interface)
    {
        case NTAG_RF_LOCK:
            /* The increment to be counted from the base address */
            BlockIncrement = BlockNo / 16;
            /* Increment the address according to offset */
            address = address + BlockIncrement;
            /* The bit order within the block */
            BitIncrement = (uint16_t)(BitIncrement << (BlockNo % 16));
            /* Reading out the already present data */
            status =  NTAG_ReadBlock( ntag, address, &data[0],  NTAG_I2C_BLOCK_SIZE);
            if(!status)
            {
                Lock = (BitIncrement > 0x80) ? (data[1] & (uint8_t)(BitIncrement >> 8)) : (data[0] & (uint8_t)BitIncrement);
                if(Lock)
                {
                    *Locked = true;
                }
                else
                {
                    *Locked = false;
                }
            }

        break;

        case NTAG_I2C_LOCK:
            /* Making the default address to be I2C address */
            address = (uint16_t)NTAG_I2C_LOCK_BLOCK_OFFSET;
            /* Per Configuration Block 64 blocks can be locked */
            BlockIncrement = BlockNo / 64;
            /* Incrementing the value of configuration block */
            address = address + BlockIncrement;
            /* The bit order within the block */
            BitIncrement = (uint16_t)(BitIncrement << ((BlockNo % 64) / 4));
            /* Reading out the already present data */
            status =  NTAG_ReadBlock( ntag, address, &data[0],  NTAG_I2C_BLOCK_SIZE);
            if(!status)
            {
                Lock = (BitIncrement > 0x80) ? (data[1] & (uint8_t)(BitIncrement >> 8)) : (data[0] & (uint8_t)BitIncrement);
                if(Lock)
                {
                    *Locked = true;
                }
                else
                {
                    *Locked = false;
                }
            }

        break;
        default:
            return true;
            break;
    }
    return status;
}


BOOL NTAG_SetI2C_NFCSILENCE(NTAG_HANDLE_T ntag,  Bool Enable)
{
    BOOL status;
    uint16_t val = 0;
    if(ntag == NULL)
    {
        return true;
    }
    /* Read the register so as not to disturb other values */
    status = NTAG_ReadRegister(ntag, NTAG_MEM_OFFSET_TAG_CONFIG0_REG, &val);
    /* Decide the value based on enabling/disabling */
    if(Enable)
    {
        val |= NTAG_MEM_I2C_NFC_SILENCE_MASK;
    }
    else
    {
        val &= NTAG_MEM_I2C_NFC_SILENCE_REVERSE;
    }

    /* Set the value to corresponding register */
    status = NTAG_WriteRegister(ntag, NTAG_MEM_OFFSET_TAG_CONFIG0_REG, NTAG_MEM_I2C_NFC_SILENCE_MASK,  val);
    return status;

}


BOOL NTAG_ReleaseI2CLocked(NTAG_HANDLE_T ntag)
{
    BOOL status;
    uint16_t val = 0;
    if(ntag == NULL)
    {
        return true;
    }
    /* Release the I2C locked, to be used mostly when WDT is not set and thus lock needs to be release mainly */
    status = NTAG_WriteRegister(ntag, NTAG_MEM_OFFSET_TAG_STATUS_REG, NTAG_MEM_I2C_IF_LOCKED_MASK,  val);
    return status;
}


BOOL NTAG_SetTransferDir(NTAG_HANDLE_T ntag, NTAG_TRANSFER_DIR_T dir)
{
    BOOL status;
    uint16_t val = 0;
    if(ntag == NULL)
    {
        return true;
    }
    /* Select the value based on the variable */
    if(dir)
    {
        val = NTAG_MEM_PT_TRANSFER_DIR_MASK;
    }
    else
    {
        val = NTAG_MEM_PT_TRANSFER_DIR_REVERSE;
    }
    /* Write the value directly to the tag config register */
    status = NTAG_WriteRegister(ntag, NTAG_MEM_OFFSET_TAG_CONFIG0_REG, NTAG_MEM_PT_TRANSFER_DIR_MASK,  val);
    return status;
}


BOOL NTAG_GetTransferDir(NTAG_HANDLE_T ntag, NTAG_TRANSFER_DIR_T *dir)
{
    BOOL status;
    uint16_t val = 0;
    if(ntag == NULL)
    {
        return true;
    }
    /* Read the tag config session register */
    status = NTAG_ReadRegister(ntag, NTAG_MEM_OFFSET_TAG_CONFIG0_REG,  &val);
    /* Derive the Transfer direction based on value at mask pos */
    if(val & NTAG_MEM_PT_TRANSFER_DIR_MASK)
    {
        *dir = RF_TO_I2C;
    }
    else
    {
        *dir = I2C_TO_RF;
    }
    return status;
}


BOOL NTAG_SetEDFunction(NTAG_HANDLE_T ntag, NTAG_ED_FUNCTIONS_T func)
{
    BOOL status;
    if(ntag == NULL)
    {
        return true;
    }
    /* Set the ED conf session register according to the input */
    status = NTAG_WriteRegister(ntag, NTAG_MEM_OFFSET_ED_CONF_REG, NTAG_MEM_ED_FUNCTION_MASK, func);
    return status;
}


BOOL NTAG_GetEDFunction(NTAG_HANDLE_T ntag, NTAG_ED_FUNCTIONS_T *func)
{
    BOOL status;
    uint16_t val = 0;
    if(ntag == NULL)
    {
        return true;
    }
    /* Read the ED conf session register */
    status = NTAG_ReadRegister(ntag, NTAG_MEM_OFFSET_ED_CONF_REG, &val);

    *func = (NTAG_ED_FUNCTIONS_T)(val & NTAG_MEM_ED_FUNCTION_MASK);

    return status;
}


BOOL NTAG_SetPHDCOnOff(NTAG_HANDLE_T ntag, BOOL on)
{
    BOOL status;
    uint16_t value = 0;
    if(ntag == NULL)
    {
        return true;
    }
    /* Select the value based on the variable */
    if(on)
    {
        value = NTAG_MEM_SRAM_PHDC_MASK;
    }
    else
    {
        value = NTAG_MEM_SRAM_PHDC_REVERSE_MASK;
    }
    /* Write the value directly to the tag config register */
    status =  NTAG_WriteRegister(ntag, NTAG_MEM_OFFSET_TAG_CONFIG0_REG, NTAG_MEM_SRAM_PHDC_MASK, value);
    return status;
}


BOOL NTAG_GetPHDCOnOff(NTAG_HANDLE_T ntag, BOOL *on)
{
    BOOL status;
    uint16_t value = 0;
    if(ntag == NULL)
    {
        return true;
    }
    /* Read the tag config session register */
    status =  NTAG_ReadRegister(ntag, NTAG_MEM_OFFSET_TAG_CONFIG0_REG, &value);
    /* Derive the PHDC configuration based on value at mask pos */
    if((value & NTAG_MEM_SRAM_PHDC_MASK) == NTAG_MEM_SRAM_PHDC_MASK)
    {
        *on = true;
    }
    else
    {
        *on = false;
    }

    return status;
}


BOOL NTAG_SetPthruOnOff(NTAG_HANDLE_T ntag, BOOL on)
{
    BOOL status;
    uint16_t value = 0;
    /* Select the value based on the variable */
    if(on)
    {
        value = NTAG_MEM_SRAM_PT_MASK;
    }
    else
    {
        value = NTAG_MEM_SRAM_PT_REVERSE_MASK;
    }
    /* Write the value directly to the tag config register */
    status =  NTAG_WriteRegister(ntag, NTAG_MEM_OFFSET_TAG_CONFIG0_REG, NTAG_MEM_SRAM_PT_MASK, value);
    return status;
}


BOOL NTAG_GetPthruOnOff(NTAG_HANDLE_T ntag, BOOL *on)
{
    BOOL status;
    uint16_t value = 0;
    /* Read the tag config session register */
    status =  NTAG_ReadRegister(ntag, NTAG_MEM_OFFSET_TAG_CONFIG0_REG, &value);
    /* Derive the pass through configuration based on value at mask pos */
    if((value & NTAG_MEM_SRAM_PT_MASK) == NTAG_MEM_SRAM_PT_MASK)
    {
        *on = true;
    }
    else
    {
        *on = false;
    }

    return status;
}


BOOL NTAG_SetSRAMMirrorOnOff(NTAG_HANDLE_T ntag, BOOL on)
{
    BOOL status;
    uint16_t value = 0;
    /* Select the value based on the variable */
    if(on)
    {
        value = NTAG_MEM_SRAM_MIRROR_MASK;
    }
    else
    {
        value = NTAG_MEM_SRAM_MIRROR_REVERSE_MASK;
    }
    /* Write the value directly to the tag config session register */
    status =  NTAG_WriteRegister(ntag, NTAG_MEM_OFFSET_TAG_CONFIG0_REG, NTAG_MEM_SRAM_MIRROR_MASK, value);
    return status;
}


BOOL NTAG_GetSRAMMirrorOnOff(NTAG_HANDLE_T ntag, BOOL *on)
{
    BOOL status;
    uint16_t value = 0;
    /* Read the TAG Config session register */
    status =  NTAG_ReadRegister(ntag, NTAG_MEM_OFFSET_TAG_CONFIG0_REG, &value);
    /* Derive the Mirror configuration based on mask */
    if((value & NTAG_MEM_SRAM_MIRROR_MASK) == NTAG_MEM_SRAM_MIRROR_MASK)
    {
        *on = true;
    }
    else
    {
        *on = false;
    }

    return status;
}


BOOL NTAG_SetLastNDEFBlock(NTAG_HANDLE_T ntag, uint16_t block)
{
    BOOL status;
    /* Set the Last Ndef block to the session register */
    status =  NTAG_WriteRegister(ntag, NTAG_MEM_OFFSET_NDEF_DATA_BLOCK_REG, 0xFFFF, block);
    return status;
}


BOOL NTAG_GetLastNDEFBlock(NTAG_HANDLE_T ntag, uint16_t *block)
{
    BOOL status;
    /* Read the Last Ndef block from the session register */
    status =  NTAG_ReadRegister(ntag, NTAG_MEM_OFFSET_NDEF_DATA_BLOCK_REG, block);
    return status;
}


BOOL NTAG_SetWatchdogTime(NTAG_HANDLE_T ntag, uint16_t time)
{
    BOOL status;
    uint8_t data[4] = {0};
    /* Read the existing block so as to not to disturb other values */
    status =  NTAG_ReadBlock(ntag, NTAG_I2C_WDT_CONF_ADDRESS, &data[0], 4);
    data[0] = (uint8_t)time;
    data[1] = (uint8_t)(time >> 8);
    /* Set the timer to the config memory address of WDT */
    status =  NTAG_WriteBlock(ntag, NTAG_I2C_WDT_CONF_ADDRESS, &data[0], 4);
    return status;
}


BOOL NTAG_GetWatchdogTime(NTAG_HANDLE_T ntag, uint16_t *time)
{
    BOOL status;
    /* Get the value of the watchdog timer by reading the value of WDT Conf reg */
    status =  NTAG_ReadRegister(ntag, NTAG_MEM_OFFSET_WDT_CONF_REG, time);
    return status;
}


