/*
*         Copyright (c), NXP Semiconductors Bangalore / India
*
*                     (C)NXP Semiconductors
*       All rights are reserved. Reproduction in whole or in part is
*      prohibited without the written consent of the copyright owner.
*  NXP reserves the right to make changes without notice at any time.
* NXP makes no warranty, expressed, implied or statutory, including but
* not limited to any implied warranty of merchantability or fitness for any
* particular purpose, or that the use will not infringe any third party patent,
* copyright or trademark. NXP must not be liable for any loss or damage
*                          arising from its use.
*/

/*
===============================================================================

Name        : test.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
===============================================================================
*/

#include "test.h"
#include "test_Osal_Apis.h"
#include "test_taskContextSwitch.h"
#include "test_taskRendezvous.h"
#include "test_IsrTaskContextSwitch.h"
#include "phPlatform.h"
/*
 * Defines
 */
#define DEMO_TASK_PRIO              1
// Stack size in no of 32 bit words. Below will reserve 800 Bytes for stack
#define MAIN_TASK_STACK_SIZE        (200)

#ifdef PHOSAL_FREERTOS_STATIC_MEM_ALLOCATION
uint32_t aMainTaskStackBuffer[MAIN_TASK_STACK_SIZE];
#else
#define aMainTaskStackBuffer                NULL
#endif

phPlatform_DataParams_t timerParams = {0};

// Below timer is used for Time Analysis
phPlatform_Timer_Struct_t *pTimerHandle;

tLogBuff timelogBuffer = {0};
/*
 * Function declaration
 */
static void Main_Test_Task (void * pDataParams);

/*******************************************************************************
**   Main Function
*******************************************************************************/
int main(void)
{
    phOsal_ThreadObj_t MainTestThreadObj = {0};

    /* Configure timer for Timing analysis */
    phPlatform_Timer_Init(&timerParams);

    phPlatform_Timer_Create(&timerParams,PH_PLATFORM_TIMER_UNIT_US,&pTimerHandle);
    phPlatform_Timer_Configure(pTimerHandle, 500000, NULL);

    /* Perform OSAL Initialization. */
    (void)phOsal_Init();

    MainTestThreadObj.pTaskName = (uint8_t *) "MainTestTask";
    MainTestThreadObj.pStackBuffer = aMainTaskStackBuffer;
    MainTestThreadObj.priority = DEMO_TASK_PRIO;
    MainTestThreadObj.stackSizeInNum = MAIN_TASK_STACK_SIZE;

    (void)phOsal_ThreadCreate(&MainTestThreadObj.ThreadHandle, &MainTestThreadObj, &Main_Test_Task, NULL);

    phOsal_StartScheduler();

    while(1) {  }
    return 0 ;
}


/*******************************************************************************
*   Main_Test_Task
*******************************************************************************/
void Main_Test_Task (void * pDataParams)
{

    DEBUG_PRINTF("\n\r Starting Tests...\n\r");

    /*Test_Osal_Apis will test the sanity testing of All Osal Apis*/
    Test_Osal_Apis();

    /* Test_Task_Context_Switch will check for if context switch between tasks
     * and between task and ISR is happening*/
    Test_Task_Context_Switch();

    /*Test of achieving Rendezvous condition between Three tasks of
     * different priority */
    Test_Task_Rendezvous();

    /* Testing ISR to Task context switch */
    Test_IsrTask_Context_Switch();

    DEBUG_PRINTF("\n\r All Tests Finished.\n\r");
    while (1){
        phOsal_ThreadDelay((phOsal_Ticks_t)OSAL_MAX_DELAY);
    }
}


