/*
 * test_taskIsrContextSwitch.c
 *
 *  Created on: Jan 25, 2017
 *      Author: nxf18375
 */


#include <test_IsrTaskContextSwitch.h>
#include <test.h>

#define TASK_PRIO               5
#define TASK_STACK_SIZE         (100)
#define TASK_STACK_BUFF         NULL

extern phPlatform_DataParams_t timerParams;

phOsal_SemObj_t TestSemObj = {0};
extern phPlatform_Timer_Struct_t *pTimerHandle;

static phPlatform_Timer_Struct_t *pTimerISRHandle;

/*
 * Test_IsrTask_Context_Switch
 * This function Measure Context Switch time from ISR to Task.
 * Function creates a timer and a Semaphores and then waits
 * for semaphore to be posted from ISR handler.
 */
void Test_IsrTask_Context_Switch(void)
{
    phOsal_TimerPeriodObj_t WaitTime = {0};
    uint32_t timeDiff = 0;

    WaitTime.period = OSAL_MAX_DELAY;
    WaitTime.unitPeriod = OS_TIMER_UNIT_MSEC;

    DEBUG_PRINTF("\n\r test_Osal  Starting Test_IsrTask_Context_Switch ()\n\r");

    /*Timer used for time logging purpose*/
    phPlatform_Timer_Reset(pTimerHandle);
    phPlatform_Timer_Start(pTimerHandle, PH_PLATFORM_TIMER_SINGLE_SHOT);

    phOsal_SemCreate(&TestSemObj.SemHandle,&TestSemObj, E_OS_SEM_OPT_SIGNALLING_SEM);

    /* Creating a timer. Will post semaphore from its ISR i.e. TIMER1_IRQHandler   */
    phPlatform_Timer_Create(&timerParams,PH_PLATFORM_TIMER_UNIT_MS,&pTimerISRHandle);
    phPlatform_Timer_Configure(pTimerISRHandle, 5, NULL);
    phPlatform_Timer_Start(pTimerISRHandle, PH_PLATFORM_TIMER_SINGLE_SHOT);

    /* Blocking on semaphore to be posted from TIMER1_IRQHandler() in file phPlatform_Port_LPCTimer*/
    phOsal_SemPend(&TestSemObj.SemHandle,WaitTime);

    /*log time to measure context switch time*/
    TIME_LOG(pTimerHandle);

    phPlatform_Timer_Stop(pTimerHandle);

    timeDiff = TIME_DIFF;
    DEBUG_PRINTF("\n\r Test_IsrTask_Context_Switch(),Context Switch time using Semaphore"
            "  = %d us\n\r", timeDiff);

    DEBUG_PRINTF("\n\r test_Osal  Test_IsrTask_Context_Switch Finished ()\n\r");
}


