/*
 * test_Osal_Apis.c
 *
 *  Created on: Jan 18, 2017
 *      Author: nxf18375
 */

#include "test.h"
#include "test_Osal_Apis.h"

/*
 * Defines needed for testing thread Apis, specific to the file
 */
#define DEMO_TASK_PRIO              3
#define DEMO_TASK_STACK_SIZE        (100)

#ifdef PHOSAL_FREERTOS_STATIC_MEM_ALLOCATION
uint32_t aDemoTaskStackBuffer[DEMO_TASK_STACK_SIZE];
#else
#define aDemoTaskStackBuffer        NULL
#endif

#define DEMO_TASK_STACK             NULL
#define DEMO_TASK_NEW_PRIORITY      5
#define DEMO_TASK_TICKS_SLEEP       2

/*
 * Static Function declaration
 */
static void Thread_Demo_Task(void  *pDataParams);
static phStatus_t Test_Thread_Apis (void);
static phStatus_t Test_Event_Apis(void);
static phStatus_t Test_Sem_Apis(void);
static phStatus_t Test_Mutex_Apis(void);
static phStatus_t Test_Timer_Apis(void);
/*
 * Variables with File scope
 */

/*******************************************************************************
*   Test_Osal_Apis
*   This function call all the Apis of exposed by OSAL as per the category.
*   By this we are testing the mapping of under-line RTOS to the OSAL Apis. This
*   Doesn't test the functional correctness of the Api.
*******************************************************************************/

void Test_Osal_Apis(void)
{
    DEBUG_PRINTF("\n\r test_Osal  Starting Test_Osal_Apis ()\n\r");

    Test_Thread_Apis();

    Test_Sem_Apis();

    Test_Mutex_Apis();

    Test_Event_Apis();

    Test_Timer_Apis();

    DEBUG_PRINTF("\n\r test_Osal  Finished Test_Osal_Apis ()\n\r");
}

/*
 * This function test the sanity of all the thread related OSAL functions. It doesn't check
 * the correctness of functionality of the APIs. It will invoke the functions and based
 * upon return value, it will determine the result
 */

phStatus_t Test_Thread_Apis(void)
{
    phStatus_t status;
    uint32_t  test_flag = 0;
    phOsal_ThreadObj_t TestThreadObj;

    TestThreadObj.pTaskName = (uint8_t *) "DemoTask";
    TestThreadObj.pStackBuffer = aDemoTaskStackBuffer;
    TestThreadObj.priority = DEMO_TASK_PRIO;
    TestThreadObj.stackSizeInNum = DEMO_TASK_STACK_SIZE;

    status = phOsal_ThreadCreate(&TestThreadObj.ThreadHandle, &TestThreadObj, &Thread_Demo_Task, &test_flag);

    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_ThreadCreate Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_ThreadCreate UnSuccessful \n\r");
    }

    status = phOsal_ThreadChangePrio(&TestThreadObj.ThreadHandle,DEMO_TASK_NEW_PRIORITY);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_ThreadChangePrio Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_ThreadChangePrio UnSuccessful \n\r");
    }

    status = phOsal_ThreadDelay(4);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_ThreadDelay Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_ThreadDelay UnSuccessful \n\r");
    }

    status = phOsal_ThreadDelete(&TestThreadObj.ThreadHandle);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_ThreadDelete Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_ThreadDelete UnSuccessful \n\r");
    }

    return PH_OSAL_SUCCESS;
}


void Thread_Demo_Task(void  *pDataParams)
{
    DEBUG_PRINTF("\n\r test_Osal  Inside Demo task \n\r");

    while(1){
        // Sleeping Infinitely, This task will be deleted
        phOsal_ThreadDelay(OSAL_MAX_DELAY);
    }
}


phStatus_t Test_Event_Apis(void)
{
    phStatus_t status;
    phOsal_EventObj_t TestEventObj = {0};
    phOsal_EventBits_t  EventReceived = 0;

    status = phOsal_EventCreate(&TestEventObj.EventHandle,&TestEventObj);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_EventCreate Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_EventCreate UnSuccessful \n\r");
    }

    status = phOsal_EventPost(&TestEventObj.EventHandle,E_OS_EVENT_OPT_NONE,0x01,NULL);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_EventPost Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_EventPost UnSuccessful \n\r");
    }

    status = phOsal_EventPend((volatile phOsal_Event_t * )&TestEventObj.EventHandle, \
            E_OS_EVENT_OPT_PEND_SET_ANY,0,0x01,&EventReceived);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_EventPend Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_EventPend UnSuccessful \n\r");
    }

    status = phOsal_EventClear(&TestEventObj.EventHandle,E_OS_EVENT_OPT_NONE,0x01, \
            &EventReceived);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_EventClear Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_EventClear UnSuccessful \n\r");
    }

    status = phOsal_EventGet(&TestEventObj.EventHandle,&EventReceived);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_EventGet Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_EventGet UnSuccessful \n\r");
    }

    status =  phOsal_EventDelete(&TestEventObj.EventHandle);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_EventDelete Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_EventDelete UnSuccessful \n\r");
    }

    return PH_OSAL_SUCCESS;

}


phStatus_t Test_Timer_Apis(void)
{
    phStatus_t status;
    phOsal_TimerObj_t TestTimeObj = {0};
    phOsal_TimerPeriodObj_t TestTimePeriod;
    uint32_t ElapsedTime = 0;

    TestTimePeriod.period = 100;
    TestTimePeriod.unitPeriod = OS_TIMER_UNIT_SEC;
    TestTimeObj.timePeriod = TestTimePeriod;

    status = phOsal_TimerCreate(&TestTimeObj.TimerHandle,&TestTimeObj);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_TimerCreate Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_TimerCreate UnSuccessful \n\r");
    }

    status = phOsal_TimerStart(&TestTimeObj.TimerHandle);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_TimerStart Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_TimerStart UnSuccessful \n\r");
    }

    status = phOsal_TimerGetCurrent(&TestTimeObj.TimerHandle,&ElapsedTime);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_TimerGetCurrent Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_TimerGetCurrent UnSuccessful \n\r");
    }

    TestTimePeriod.period = 1;
    TestTimeObj.timePeriod = TestTimePeriod;
    status = phOsal_TimerModify(&TestTimeObj.TimerHandle,&TestTimeObj);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_TimerModify Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_TimerModify UnSuccessful \n\r");
    }

    status = phOsal_TimerStop(&TestTimeObj.TimerHandle);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_TimerStop Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_TimerStop UnSuccessful \n\r");
    }

    status = phOsal_TimerDelete(&TestTimeObj.TimerHandle);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_TimerDelete Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_TimerDelete UnSuccessful \n\r");
    }

    return PH_OSAL_SUCCESS;
}

phStatus_t Test_Sem_Apis(void){

    phStatus_t status;

    phOsal_SemObj_t TestSemObj = {0};
    phOsal_TimerPeriodObj_t WaitTime = {0};

    status = phOsal_SemCreate(&TestSemObj.SemHandle,&TestSemObj, E_OS_SEM_OPT_BINARY_SEM);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_SemCreate Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_SemCreate UnSuccessful \n\r");
    }

    status = phOsal_SemPost(&TestSemObj.SemHandle,E_OS_SEM_OPT_BINARY_SEM);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_SemPost Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_SemPost UnSuccessful \n\r");
    }

    status = phOsal_SemPend(&TestSemObj.SemHandle,WaitTime);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_SemPend Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_SemPend UnSuccessful \n\r");
    }

    status = phOsal_SemDelete(&TestSemObj.SemHandle);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_SemDelete Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_SemDelete UnSuccessful \n\r");
    }

    return PH_OSAL_SUCCESS;

}


phStatus_t Test_Mutex_Apis(void){

    phStatus_t status;
    phOsal_MutexObj_t TestMutexObj = {0};
    phOsal_TimerPeriodObj_t TestPeriod = {0};

    status = phOsal_MutexCreate(&TestMutexObj.MutexHandle,&TestMutexObj);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_MutexCreate Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_MutexCreate UnSuccessful \n\r");
    }

    status = phOsal_MutexLock(&TestMutexObj.MutexHandle,TestPeriod);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_MutexLock Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_MutexLock UnSuccessful \n\r");
    }

    status = phOsal_MutexUnLock(&TestMutexObj.MutexHandle);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_MutexUnLock Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_MutexUnLock UnSuccessful \n\r");
    }

    status = phOsal_MutexDelete(&TestMutexObj.MutexHandle);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  phOsal_MutexDelete Successful \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  phOsal_MutexDelete UnSuccessful \n\r");
    }

    return PH_OSAL_SUCCESS;
}

