/*
 * test_taskContextSwitch.c
 *
 *  Created on: Jan 20, 2017
 *      Author: nxf18375
 */

#include "test.h"
#include "test_taskContextSwitch.h"

/*
 * Defines specific to the file
 */
#define HIGH_TASK_PRIO              5
#define LOW_TASK_PRIO               4
#define TASK_STACK_SIZE             (120)

#ifdef PHOSAL_FREERTOS_STATIC_MEM_ALLOCATION
uint32_t aHighTaskStackBuffer[TASK_STACK_SIZE];
uint32_t aLowTaskStackBuffer[TASK_STACK_SIZE];
#else
#define aHighTaskStackBuffer        NULL
#define aLowTaskStackBuffer         NULL
#endif

#define TASK_STACK_BUFF             NULL

#define SIGNAL_HIGH_TASK 0x01

extern phPlatform_Timer_Struct_t *pTimerHandle;

void Demo_High_Task (void *params);

void Demo_Low_Task (void *params);

void create_tasks(phOsal_ThreadObj_t * highPriorityThread, phOsal_ThreadObj_t * lowPriorityThread, \
                    uint32_t *Resourceflag);


static phOsal_SemObj_t TestSemObj = {0};
static phOsal_EventObj_t TestEventObj = {0};
static phOsal_EventBits_t  EventReceived = 0;


/*
 * Test_Task_Context_Switch
 * This will create  two tasks, One with higher priority and
 * another with lower priority.
 *
 * From lower priority task we will signal higher priority
 * task and that should run immediately. Before returning readout the
 * time needed for the context switch
 */
void Test_Task_Context_Switch()
{
    phOsal_ThreadObj_t HighThreadObj;
    phOsal_ThreadObj_t LowThreadObj;
    uint32_t  TestFlag = 0;
    uint32_t timeDiff = 0;

    DEBUG_PRINTF("\n\r test_Osal  Starting Test_Task_Context_Switch ()\n\r");
    phPlatform_Timer_Reset(pTimerHandle);
    phPlatform_Timer_Start(pTimerHandle, PH_PLATFORM_TIMER_SINGLE_SHOT);

    {
        phOsal_SemCreate(&TestSemObj.SemHandle,&TestSemObj, E_OS_SEM_OPT_SIGNALLING_SEM);

        create_tasks(&HighThreadObj,&LowThreadObj,&TestFlag);

        phOsal_ThreadDelete(&HighThreadObj.ThreadHandle);
        phOsal_ThreadDelete(&LowThreadObj.ThreadHandle);
        phOsal_SemDelete(&TestSemObj.SemHandle);
    }

    timeDiff = TIME_DIFF;
    DEBUG_PRINTF("\n\r Test_Task_Context_Switch(), time consumed in Context Switch using Semaphore"
            "  = %d us\n\r", timeDiff);
    {
        phOsal_EventCreate(&TestEventObj.EventHandle,&TestEventObj);

        TestFlag = 1;
        create_tasks(&HighThreadObj,&LowThreadObj,&TestFlag);

        phOsal_ThreadDelete(&HighThreadObj.ThreadHandle);
        phOsal_ThreadDelete(&LowThreadObj.ThreadHandle);
        phOsal_EventDelete(&TestEventObj.EventHandle);
    }

    timeDiff = TIME_DIFF;
    DEBUG_PRINTF("\n\r Test_Task_Context_Switch(), time consumed in Context Switch using Events"
            "  = %d us\n\r", timeDiff);

    // stopping previously running platform timer used to timeMesurement of context Switch
    phPlatform_Timer_Stop(pTimerHandle);

    DEBUG_PRINTF("\n\r test_Osal  Finished Test_Task_Context_Switch ()\n\r");
}



void Demo_High_Task (void *params)
{
    phOsal_TimerPeriodObj_t WaitTime = {0};
    WaitTime.period = OSAL_MAX_DELAY;
    WaitTime.unitPeriod = OS_TIMER_UNIT_MSEC;

    uint32_t resourceFlag = * (uint32_t *)params;

    DEBUG_PRINTF("\n\r test_Osal High Priority Task\n\r");
    while(1)
    {
        if (resourceFlag == 0)
        {
            DEBUG_PRINTF("\n\r test_Osal High Priority Task before Sem_Pend\n\r");

            phOsal_SemPend(&TestSemObj.SemHandle,WaitTime);

            // Store the time
            TIME_LOG(pTimerHandle);

            DEBUG_PRINTF("\n\r test_Osal High Priority Task after Sem_Pend\n\r");

        }
        else{
            DEBUG_PRINTF("\n\r test_Osal High Priority Task before Event_Pend\n\r");

            phOsal_EventPend((volatile phOsal_Event_t * )&TestEventObj.EventHandle, \
                    E_OS_EVENT_OPT_PEND_SET_ALL,OSAL_MAX_DELAY, \
                    SIGNAL_HIGH_TASK,&EventReceived);
            // Store the time
            TIME_LOG(pTimerHandle);

            DEBUG_PRINTF("\n\r test_Osal High Priority Task after Event_Pend\n\r");

        }

        phOsal_ThreadDelay(OSAL_MAX_DELAY);
    }
}


void Demo_Low_Task (void *params)
{
    uint32_t resourceFlag = * (uint32_t *)params;

    DEBUG_PRINTF("\n\r test_Osal Low Priority Task\n\r");

    while(1)
    {
        if (resourceFlag == 0)
        {
            DEBUG_PRINTF("\n\r test_Osal Low Priority Task before Sem_Post\n\r");

            TIME_LOG(pTimerHandle);

            phOsal_SemPost(&TestSemObj.SemHandle,E_OS_SEM_OPT_SIGNALLING_SEM);

            DEBUG_PRINTF("\n\r test_Osal Low Priority Task after Sem_Post\n\r");

        }
        else{
            DEBUG_PRINTF("\n\r test_Osal Low Priority Task before Event_Post\n\r");

            TIME_LOG(pTimerHandle);

            phOsal_EventPost(&TestEventObj.EventHandle,E_OS_EVENT_OPT_NONE,SIGNAL_HIGH_TASK,NULL);

            DEBUG_PRINTF("\n\r test_Osal Low Priority Task after Event_Post\n\r");
        }

        phOsal_ThreadDelay(OSAL_MAX_DELAY);
    }
}


void create_tasks(phOsal_ThreadObj_t * highPriorityThread, phOsal_ThreadObj_t * lowPriorityThread,
                uint32_t *Resourceflag)
{
    highPriorityThread->pTaskName = (uint8_t *) "HighPrioTask";
    highPriorityThread->pStackBuffer = aHighTaskStackBuffer;
    highPriorityThread->priority = HIGH_TASK_PRIO;
    highPriorityThread->stackSizeInNum = TASK_STACK_SIZE;

    phOsal_ThreadCreate(&highPriorityThread->ThreadHandle, highPriorityThread, &Demo_High_Task, Resourceflag);

    lowPriorityThread->pTaskName = (uint8_t *) "LowPrioTask";
    lowPriorityThread->pStackBuffer = aLowTaskStackBuffer;
    lowPriorityThread->priority = LOW_TASK_PRIO;
    lowPriorityThread->stackSizeInNum = TASK_STACK_SIZE;

    phOsal_ThreadCreate(&lowPriorityThread->ThreadHandle, lowPriorityThread, &Demo_Low_Task, Resourceflag);

}
