/*
 * test_taskRendezvous.c
 *
 *  Created on: Jan 20, 2017
 *      Author: nxf18375
 */

#include "test.h"
#include "test_taskRendezvous.h"

#define CHILD_1_PRIO                2
#define CHILD_2_PRIO                3
#define CHILD_3_PRIO                4
#define TASK_STACK_SIZE             (100)

#ifdef PHOSAL_FREERTOS_STATIC_MEM_ALLOCATION
uint32_t aChild_1_StackBuffer[TASK_STACK_SIZE];
uint32_t aChild_2_StackBuffer[TASK_STACK_SIZE];
uint32_t aChild_3_StackBuffer[TASK_STACK_SIZE];
#else
#define aChild_1_StackBuffer        NULL
#define aChild_2_StackBuffer        NULL
#define aChild_3_StackBuffer        NULL
#endif


#define TASK_STACK_BUFF             NULL

#define TEST_OSAL_WAIT_CHILD_1  0x01
#define TEST_OSAL_WAIT_CHILD_2  0x02
#define TEST_OSAL_WAIT_CHILD_3  0x04
#define TEST_OSAL_WAIT_ALL      0x0F

void Child_1_Task(void *params);
void Child_2_Task(void *params);
void Child_3_Task(void *params);

static phOsal_EventObj_t TestEventObj = {0};
static phOsal_EventObj_t TestEventObj_1 = {0};
static phOsal_EventBits_t  EventReceived = 0;

/*
 * Test_Task_Rendezvous
 * This function two child tasks, of same priority but higher
 * than Parent.
 *
 * Child tasks wait for signal from Parent to proceed. Each tasks
 * once finished signals the Parent Back
 */
void Test_Task_Rendezvous(void)
{
    uint32_t  TestFlag = 0;
    phStatus_t status;

    phOsal_ThreadObj_t ChildOneThreadObj = {0};
    phOsal_ThreadObj_t ChildTwoThreadObj = {0};
    phOsal_ThreadObj_t ChildThreeThreadObj = {0};

    DEBUG_PRINTF("\n\r test_Osal  Starting Test_Task_Rendezvous ()\n\r");

    phOsal_EventCreate(&TestEventObj.EventHandle,&TestEventObj);
    phOsal_EventCreate(&TestEventObj_1.EventHandle,&TestEventObj_1);

    // Create two tasks
    ChildOneThreadObj.pTaskName = (uint8_t *) "Child_1";
    ChildOneThreadObj.pStackBuffer = aChild_1_StackBuffer;
    ChildOneThreadObj.priority = CHILD_1_PRIO;
    ChildOneThreadObj.stackSizeInNum = TASK_STACK_SIZE;
    phOsal_ThreadCreate(&ChildOneThreadObj.ThreadHandle, &ChildOneThreadObj, &Child_1_Task, &TestFlag);

    ChildTwoThreadObj.pTaskName = (uint8_t *) "Child_2";
    ChildTwoThreadObj.pStackBuffer = aChild_2_StackBuffer;
    ChildTwoThreadObj.priority = CHILD_2_PRIO;
    ChildTwoThreadObj.stackSizeInNum = TASK_STACK_SIZE;
    phOsal_ThreadCreate(&ChildTwoThreadObj.ThreadHandle, &ChildTwoThreadObj, &Child_2_Task, &TestFlag);

    ChildThreeThreadObj.pTaskName = (uint8_t *) "Child_3";
    ChildThreeThreadObj.pStackBuffer = aChild_3_StackBuffer;
    ChildThreeThreadObj.priority = CHILD_3_PRIO;
    ChildThreeThreadObj.stackSizeInNum = TASK_STACK_SIZE;
    phOsal_ThreadCreate(&ChildThreeThreadObj.ThreadHandle, &ChildThreeThreadObj, &Child_3_Task, &TestFlag);

    // wait for signal from the child tasks
    status = phOsal_EventPend((volatile phOsal_Event_t * )&TestEventObj.EventHandle, \
                E_OS_EVENT_OPT_PEND_SET_ALL,OSAL_MAX_DELAY, \
                (TEST_OSAL_WAIT_CHILD_1 | TEST_OSAL_WAIT_CHILD_2 | TEST_OSAL_WAIT_CHILD_3), \
                &EventReceived);
    if (status == PH_OSAL_SUCCESS)
    {
        DEBUG_PRINTF("\n\r test_Osal  Test_Task_Rendezvous CHILDS Initialized \n\r");
    }
    else{
        DEBUG_PRINTF("\n\r test_Osal  Test_Task_Rendezvous CHILDS Initialized UnSuccessful \n\r");
    }
    phOsal_ThreadDelay(2);

    phOsal_EventPost(&TestEventObj_1.EventHandle,E_OS_EVENT_OPT_NONE,TEST_OSAL_WAIT_ALL,NULL);

    phOsal_ThreadDelete(&ChildOneThreadObj.ThreadHandle);
    phOsal_ThreadDelete(&ChildTwoThreadObj.ThreadHandle);
    phOsal_ThreadDelete(&ChildThreeThreadObj.ThreadHandle);

    DEBUG_PRINTF("\n\r test_Osal  Finished Test_Task_Rendezvous ()\n\r");
}

void Child_1_Task(void *params)
{
    while(1)
    {
        DEBUG_PRINTF("\n\r test_Osal  Starting Child_1_Task \n\r");

        phOsal_ThreadDelay(10);

        phOsal_EventPost(&TestEventObj.EventHandle,E_OS_EVENT_OPT_NONE,TEST_OSAL_WAIT_CHILD_1,NULL);

        DEBUG_PRINTF("\n\r test_Osal  Child_1_Task Waiting \n\r");

        phOsal_EventPend((volatile phOsal_Event_t * )&TestEventObj_1.EventHandle, \
                    E_OS_EVENT_OPT_PEND_SET_ALL,OSAL_MAX_DELAY, \
                    TEST_OSAL_WAIT_ALL, &EventReceived);

        DEBUG_PRINTF("\n\r test_Osal  Child_1_Task Wake \n\r");

        phOsal_ThreadDelay(OSAL_MAX_DELAY);
    }
}

void Child_2_Task(void *params)
{
    while(1)
    {
        DEBUG_PRINTF("\n\r test_Osal  Starting Child_2_Task \n\r");

        phOsal_ThreadDelay(5);

        phOsal_EventPost(&TestEventObj.EventHandle,E_OS_EVENT_OPT_NONE,TEST_OSAL_WAIT_CHILD_2,NULL);

        DEBUG_PRINTF("\n\r test_Osal  Child_2_Task Waiting \n\r");

        phOsal_EventPend((volatile phOsal_Event_t * )&TestEventObj_1.EventHandle, \
                    E_OS_EVENT_OPT_PEND_SET_ALL,OSAL_MAX_DELAY, \
                    TEST_OSAL_WAIT_ALL, &EventReceived);

        DEBUG_PRINTF("\n\r test_Osal  Child_2_Task Wake \n\r");

        phOsal_ThreadDelay(OSAL_MAX_DELAY);
    }
}

void Child_3_Task(void *params)
{
    while(1)
    {
        DEBUG_PRINTF("\n\r test_Osal  Starting Child_3_Task \n\r");

        phOsal_ThreadDelay(5);

        phOsal_EventPost(&TestEventObj.EventHandle,E_OS_EVENT_OPT_NONE,TEST_OSAL_WAIT_CHILD_3,NULL);

        DEBUG_PRINTF("\n\r test_Osal  Child_3_Task Waiting \n\r");

        phOsal_EventPend((volatile phOsal_Event_t * )&TestEventObj_1.EventHandle, \
                    E_OS_EVENT_OPT_PEND_SET_ALL,OSAL_MAX_DELAY, \
                    TEST_OSAL_WAIT_ALL, &EventReceived);

        DEBUG_PRINTF("\n\r test_Osal  Child_3_Task Wake \n\r");

        phOsal_ThreadDelay(OSAL_MAX_DELAY);
    }
}

