/*
 * @brief NXP LPCXpresso 11U14 Sysinit file
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

 #include "board.h"
 #include "string.h"

/* The System initialization code is called prior to the application and
   initializes the board for run-time operation. Board initialization
   includes clock setup and default pin muxing configuration. */

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

/* IOCON pin definitions for pin muxing */
typedef struct {
	uint32_t port : 8;			/* Pin port */
	uint32_t pin : 8;			/* Pin number */
	uint32_t modefunc : 16;		/* Function and mode */
} PINMUX_GRP_T;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/




/*****************************************************************************
 * Private functions
 ****************************************************************************/

/* Setup system clocking */
STATIC void SystemSetupClocking(void)
{
	volatile int i;

	/* Powerup extern/main oscillator (12MHz) */
	Chip_SYSCTL_PowerUp(SYSCTL_POWERDOWN_SYSOSC_PD);
	/* Wait 200us for OSC to be stablized, no status indication, dummy wait. */
	for (i = 0; i < 0x100; i++) {}

	/* Set clock source to the PLL_IN clock source (12MHz)*/
	Chip_Clock_SetMainClockSource(SYSCTL_MAINCLKSRC_PLLIN);
	/* Set PLL_IN clk to external/main oscillator (12MHz)*/
	Chip_Clock_SetSystemPLLSource(SYSCTL_PLLCLKSRC_MAINOSC);

	/* Divide the clock down to (12MHz/1 = 12MHz) */
	Chip_Clock_SetSysClockDiv(1);

//	/* Setup FLASH access to 3 clocks */
//	Chip_FMC_SetFLASHAccess(FLASHTIM_50MHZ_CPU);
}


/* Sets up system pin muxing */
STATIC void SystemSetupMuxing(void)
{
	/* Enable IOCON clock */
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_IOCON);


	/* Pin muxing table, only items that need changing from their default pin state are in this table. */
	Chip_IOCON_PinMuxSet(LPC_IOCON, 0,  1,  (IOCON_FUNC1 | IOCON_MODE_INACT));		/* PIO0_1 used for CLKOUT */
	Chip_IOCON_PinMuxSet(LPC_IOCON, 0,  2,  (IOCON_FUNC1 | IOCON_MODE_INACT));		/* PIO0_2 used for SSEL */
//	Chip_IOCON_PinMuxSet(LPC_IOCON, 0,  3,  (IOCON_FUNC1 | IOCON_MODE_INACT));		/* PIO0_3 used for USB_VBUS */
	Chip_IOCON_PinMuxSet(LPC_IOCON, 0,  4,  (IOCON_FUNC1 | IOCON_SFI2C_EN));		/* PIO0_4 used for SCL */
	Chip_IOCON_PinMuxSet(LPC_IOCON, 0,  5,  (IOCON_FUNC1 | IOCON_SFI2C_EN));		/* PIO0_5 used for SDA */
//	Chip_IOCON_PinMuxSet(LPC_IOCON, 0,  6,  (IOCON_FUNC1 | IOCON_MODE_INACT));		/* PIO0_6 used for USB_CONNECT */
//	Chip_IOCON_PinMuxSet(LPC_IOCON, 0,  11, (IOCON_FUNC1 | IOCON_ADMODE_EN | IOCON_FILT_DIS));	/* PIO0_11 used for AD0 */
//	Chip_IOCON_PinMuxSet(LPC_IOCON, 0,  18, (IOCON_FUNC1 | IOCON_MODE_INACT));		/* PIO0_18 used for RXD */
//	Chip_IOCON_PinMuxSet(LPC_IOCON, 0,  19, (IOCON_FUNC1 | IOCON_MODE_INACT));		/* PIO0_19 used for TXD */

		/* I2C0 */
//	Chip_IOCON_PinMuxSet(LPC_IOCON, 0,  4,  IOCON_FUNC1); //ABOVE
//	Chip_IOCON_PinMuxSet(LPC_IOCON, 0,  5,  IOCON_FUNC1);
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/* Set up and initialize hardware prior to call to main */
void Board_SystemInit(void)
{
	/* Setup system clocking and muxing */
	SystemSetupClocking();
	SystemSetupMuxing();

	/* Enable I/OH SRAM (SRAM1) */
//	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_RAM1);
}
