/*
 * ntag5.h
 *
 *  Created on: 17.01.2020
 *      Author: nxf52188
 */

#ifndef INC_NTAG5_H_
#define INC_NTAG5_H_

#include <stdint.h>

#define NTAG5_SRAM_BLOCKS              0x40
#define NTAG5_SRAM_BLOCK_SIZE          4
#define NTAG5_SRAM_SIZE                (NTAG5_SRAM_BLOCKS*NTAG5_SRAM_BLOCK_SIZE)
#define NTAG5_MEM_BLOCK_START_SRAM  0x2000
#define NTAG5_MEM_BLOCK_END_SRAM    0x203F


#define NTAG_MEM_OFFSET_TAG_STATUS_REG            0x00
#define NTAG_MEM_OFFSET_TAG_CONFIG0_REG           0x01
#define NTAG_MEM_OFFSET_TAG_CONFIG1_REG           0x02
#define NTAG_MEM_OFFSET_NDEF_DATA_BLOCK_REG       0x03
#define NTAG_MEM_OFFSET_PWM_GPIO_CONFIG_REG       0x04
#define NTAG_MEM_OFFSET_PWM0_ON_REG               0x05
#define NTAG_MEM_OFFSET_PWM0_OFF_REG              0x06
#define NTAG_MEM_OFFSET_PWM1_ON_REG               0x07
#define NTAG_MEM_OFFSET_PWM1_OFF_REG              0x08
#define NTAG_MEM_OFFSET_WDT_CONF_REG              0x09
#define NTAG_MEM_OFFSET_WDT_EN_REG                0x0A
#define NTAG_MEM_OFFSET_EH_CONF_REG               0x0B
#define NTAG_MEM_OFFSET_ED_CONF_REG               0x0C
#define NTAG_MEM_OFFSET_I2C_SLAVE_CONF_REG        0x0D
#define NTAG_MEM_OFFSET_RESET_GEN_REG             0x0E
#define NTAG_MEM_OFFSET_ED_INTR_CLEAR_REG         0x0F
#define NTAG_MEM_OFFSET_I2CM_SLAVE_ADDR_REG       0x10
#define NTAG_MEM_OFFSET_I2CM_DATA_LEN_REG         0x11
#define NTAG_MEM_OFFSET_ALM_STATUS_REG            0x12
#define NTAG_MEM_OFFSET_I2CM_STATUS_REG           0x13

static const uint16_t SessionRegister[][3] =
{
        /* Block Address, Byte Offset, Byte Width */
        {0x10A0,          0,              2},     /* Tag Status Reg */
        {0x10A1,          0,              2},     /* Tag Config Reg 0 */
        {0x10A1,          2,              2},     /* Tag Config Reg 1 */
        {0x10A2,          0,              2},     /* Ndef Data Block Reg */
        {0x10A3,          0,              2},     /* PWM GPIO Config Reg */
        {0x10A4,          0,              2},     /* PWM0 ON Reg */
        {0x10A4,          2,              2},     /* PWM0 OFF Reg */
        {0x10A5,          0,              2},     /* PWM1 ON Reg */
        {0x10A5,          2,              2},     /* PWM1 OFF Reg */
        {0x10A6,          0,              2},     /* WDT CONF Reg */
        {0x10A6,          2,              1},     /* WDT EN Reg */
        {0x10A7,          0,              1},     /* EH CONF Reg */
        {0x10A8,          0,              1},     /* ED CONF Reg */
        {0x10A9,          0,              2},     /* I2C SLAVE CONF Reg */
        {0x10AA,          0,              1},     /* RESET GEN Reg */
        {0x10AB,          0,              1},     /* ED INTR CLEAR Reg */
        {0x10AC,          0,              1},     /* I2CM SLAVE ADDR Reg */
        {0x10AC,          1,              1},     /* I2CM DATA LEN Reg */
        {0x10AC,          2,              1},     /* ALM Status Reg */
        {0x10AD,          0,              1}      /* I2CM STATUS Reg */
};


uint8_t sramBuff[NTAG5_SRAM_SIZE];

uint16_t ntag5_ReadRegister(uint8_t reg, uint16_t *val);
uint16_t ntag5_WriteRegister(uint8_t reg, uint16_t mask, uint16_t val);
uint16_t ntag5_ReadSRAM();
uint16_t ntag5_WriteSRAM();

void ntag5_Clear_I2C_Locked();


#endif /* INC_NTAG5_H_ */
