/*
 * i2c.c
 *
 *  Created on: 17.01.2020
 *      Author: nxf52188
 */
#include "i2c.h"


void Init_I2C_PinMux(void)
{
#if (defined(BOARD_NXP_XPRESSO_11U14) || defined(BOARD_NGX_BLUEBOARD_11U24) || defined(BOARD_NXP_XPRESSO_11U37H))
	Chip_SYSCTL_PeriphReset(RESET_I2C0);
	Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 4, IOCON_FUNC1 | I2C_FASTPLUS_BIT);
	Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 5, IOCON_FUNC1 | I2C_FASTPLUS_BIT);
#elif (defined(BOARD_NXP_XPRESSO_11C24) || defined(BOARD_MCORE48_1125))
	Chip_SYSCTL_PeriphReset(RESET_I2C0);
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO0_4, IOCON_FUNC1 | I2C_FASTPLUS_BIT);
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO0_5, IOCON_FUNC1 | I2C_FASTPLUS_BIT);
#else
	#error "No Pin Muxing defined for I2C operation"
#endif
}

/* State machine handler for I2C0 and I2C1 */
void i2c_state_handling(I2C_ID_T id)
{
	if (Chip_I2C_IsMasterActive(id)) {
		Chip_I2C_MasterStateHandler(id);
	}
	else {
		Chip_I2C_SlaveStateHandler(id);
	}
}


/* Set I2C mode to polling/interrupt */
void i2c_set_mode(I2C_ID_T id, int polling)
{
	if (!polling) {
		Chip_I2C_SetMasterEventHandler(id, Chip_I2C_EventHandler);
		NVIC_EnableIRQ(I2C0_IRQn);
	}
	else {
		NVIC_DisableIRQ(I2C0_IRQn);
		Chip_I2C_SetMasterEventHandler(id, Chip_I2C_EventHandlerPolling);
	}
}

/**
 * @brief	I2C Interrupt Handler
 * @return	None
 */
void I2C_IRQHandler(void)
{
	i2c_state_handling(DEFAULT_I2C);
}

uint16_t ntag5_i2c_Transfer(uint8_t *txBuff, uint16_t txSize, uint8_t *rxBuff, uint16_t rxSize)
{
	static I2C_XFER_T xfer;

	/* Setup I2C transfer record */
	xfer.slaveAddr = NTAG5_I2C_ADDR & 0xFF;
	xfer.txBuff = txBuff;
	xfer.txSz = txSize;
	xfer.rxBuff = rxBuff;
	xfer.rxSz = rxSize;
	xfer.status = 0;

	Chip_I2C_MasterTransfer(DEFAULT_I2C, &xfer);
	//DEBUG_PRINTF("Master transfer : %s\r\n", xfer.status == I2C_STATUS_DONE ? "SUCCESS" : "FAILURE");
	//DEBUG_PRINTF("Received %d bytes from slave 0x%02X\r\n", rxSize - xfer.rxSz, NTAG5_I2C_ADDR);

//		1 I2C_STATUS_DONE,	    /**< Transfer done successfully */
//		2 I2C_STATUS_NAK,		/**< NAK received during transfer */
//		3 I2C_STATUS_ARBLOST,	/**< Aribitration lost during transfer */
//		4 I2C_STATUS_BUSERR,	/**< Bus error in I2C transfer */
//		5 I2C_STATUS_BUSY,	    /**< I2C is busy doing transfer */
//		6 I2C_STATUS_SLAVENAK,  /**< NAK received after SLA+W or SLA+R */
	return (xfer.status != I2C_STATUS_DONE);
}


/* Function that probes all available slaves connected to an I2C bus */
uint8_t i2c_probe_slave(I2C_ID_T i2c, int addr)
{
	uint8_t ch[2];
	/* Address 0x48 points to LM75AIM device which needs 2 bytes be read */
	if (Chip_I2C_MasterRead(i2c, addr, ch, 1 + (addr == 0x48)) > 0) {
		return false;
	}
	return true;
}

