/*
 * @brief I2C example.
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2020
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */
#include "ntag5.h"
#include "i2c.h"
#include <string.h>

uint16_t ntag5_ReadRegister(uint8_t reg, uint16_t *val)
{
    uint8_t data[4] = {0x00};
    uint8_t Rxdata[2] ={0x00};

    data[0] = (uint8_t)(SessionRegister[reg][0] >> 8);
    data[1] = (uint8_t)(SessionRegister[reg][0]);
    data[2] = (uint8_t)(SessionRegister[reg][1]);

    /* Read the Register byte */
    if (!ntag5_i2c_Transfer(data, 3, &Rxdata[0], 1))
    {
    	if(SessionRegister[reg][2] > 1)
		{
    		/* Two bytes to read */
			data[2] = (uint8_t)(SessionRegister[reg][1] + 1);
			/* Read the Register byte */
			if (!ntag5_i2c_Transfer(data, 3, &Rxdata[1], 1))
			{
				*val = (uint16_t)((uint16_t)(Rxdata[1] << 8) | Rxdata[0]);
			} else {
				return true;
			}
		}
		else
		{
			*val = Rxdata[0];
		}

    } else {
		return true;
    }

    return false;
}

uint16_t ntag5_WriteRegister(uint8_t reg, uint16_t mask, uint16_t val)
{
    uint8_t data[5] = {0x00};
    data[0] = (uint8_t)(SessionRegister[reg][0] >> 8);
    data[1] = (uint8_t)(SessionRegister[reg][0]);
    data[2] = (uint8_t)(SessionRegister[reg][1]);
    data[3] = (uint8_t)(mask);
    data[4] = (uint8_t)(val);

    if (ntag5_i2c_Transfer(data, 5, NULL, 0))
    {
		return true;
    }
	if(SessionRegister[reg][2] > 1)
	{
		data[2] = (uint8_t)(SessionRegister[reg][1] + 1);
		data[3] = (uint8_t)(mask >> 8);
		data[4] = (uint8_t)(val >> 8);

		/* Read the Register byte */
		if (ntag5_i2c_Transfer(data, 5, NULL, 0))
		{
			return true;
		}
	}

    return false;
}


uint16_t ntag5_ReadSRAM()
{
//    uint16_t wRxLength = 0;
    uint8_t data[2];
    uint16_t status = 0;
//    uint8_t blocking = 1;

    /* Assign the correct address */
    data[0] = (uint8_t)(NTAG5_MEM_BLOCK_START_SRAM >> 8);
    data[1] = (uint8_t)(NTAG5_MEM_BLOCK_START_SRAM);


	status = ntag5_i2c_Transfer(data, 2, sramBuff, NTAG5_SRAM_SIZE);


    if(status)
    {
        return true; /* failed */
    }
    return false; /* SUCCESS */
}

uint16_t ntag5_WriteSRAM(uint8_t* SRAM)
{
    static uint8_t data[2 + 256];
    uint16_t status = 0;

    /* Assign the correct address */
    data[0] = (uint8_t)(NTAG5_MEM_BLOCK_START_SRAM >> 8);
    data[1] = (uint8_t)(NTAG5_MEM_BLOCK_START_SRAM);
    memcpy(data+2, SRAM, 256);

	status = ntag5_i2c_Transfer(data, sizeof(data), NULL, 0);


    if(status)
    {
        return true; /* failed */
    }
    return false; /* SUCCESS */
}



void ntag5_Clear_I2C_Locked()
{

	uint8_t aStatusReg[2];
	uint16_t error_status = ntag5_ReadRegister(NTAG_MEM_OFFSET_TAG_STATUS_REG, (uint16_t*)(&aStatusReg[0]));
	if(error_status) {
		//DEBUG_PRINTF("Error receiving status register!\n");
	} else if(aStatusReg[1] & 0b10) {
		/* I2C_IF_LOCKED ... reset to NFC_LOCKED*/
		ntag5_ReadSRAM();
		// NTAG_ReadRegister(NTAG_MEM_OFFSET_TAG_STATUS_REG, (uint16_t*)(&aStatusReg[0]));
	}
}


