/*
 * @brief I2C example.
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2020
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include <stdlib.h>
#include <string.h>
#include "board.h"
#include <sysctl_11xx.h>
#include "ntag5.h"
#include "i2c.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/
//#define USE_TIMER 1

#define LED0_RED      0 /* Red left: near NXPLOGO */
#define LED1_RED      1 /* Red second */
#define LED2_RED      2 /* Red third  */
#define LED3_RED      3 /* Red fourth */
#define RGB_LED_RED   4 /* Red   RGB  */
#define RGB_LED_GREEN 5/* Green RGB  */
#define RGB_LED_BLUE  6/* Blue  RGB  */


#define ED_PIN			1 /* GPIO pin  number mapped to PININT */
#define ED_PIN_PORT		0 /* GPIO port number mapped to PININT */
#define ED_PIN_INDEX	0 /* PININT index used for GPIO mapping */
#define PININT_IRQ_HANDLER	FLEX_INT0_IRQHandler /* PININT IRQ function name */
#define PININT_NVIC_NAME		PIN_INT0_IRQn /* PININT NVIC interrupt name */

typedef enum direction{
	rf_i2c = 0,
	i2c_rf = 1,
} Direction;
static volatile Direction g_Direction = rf_i2c;
static volatile bool rf_sram_written = false; /* Check if SRAM has been written via RF */
static volatile bool rf_sram_read = false; /* Check if SRAM has been read via RF */

static volatile bool i2c_rf_interrupt = false; /* This is set when the I2C to RF interrupt is triggered via ED pin */
static volatile bool rf_i2c_interrupt = false; /* This is set when the RF to I2C interrupt is triggered via ED pin */
static volatile bool tag_configured = false;

uint8_t read_loop_counter, write_loop_counter;


uint8_t gSRamData[256]; /* Variable used to read content of the SRAM via I2C */

/* Variable used to write the SRAM with these fixed data */
uint8_t gSRamDataFixed[256]  =
{
		'N','X','P',' ','S','e','m','i','c','o','n','d','u','c','t','o','r','s',' ','c','r','e','a','t','e','s',' ','s','o',
		'l','u','t','i','o','n','s',' ','t','h','a','t',' ','e','n','a','b','l','e',' ','s','e','c','u','r','e',' ','c','o',
		'n','n','e','c','t','i','o','n','s',' ','f','o','r',' ','a',' ','s','m','a','r','t','e','r',' ','w','o','r','l','d',
		'.',' ','B','u','i','l','d','i','n','g',' ','o','n',' ','i','t','s',' ','e','x','p','e','r','t','i','s','e',' ','i',
		'n',' ','H','i','g','h',' ','P','e','r','f','o','r','m','a','n','c','e',' ','M','i','x','e','d',' ','S','i','g','n',
		'a','l',' ','e','l','e','c','t','r','o','n','i','c','s',',',' ','N','X','P',' ','i','s',' ','d','r','i','v','i','n',
		'g',' ','i','n','n','o','v','a','t','i','o','n',' ','i','n',' ','t','h','e',' ','a','u','t','o','m','o','t','i','v',
		'e',',',' ','i','d','e','n','t','i','f','i','c','a','t','i','o','n',' ','a','n','d',' ','m','o','b','i','l','e',' ',
		'i','n','d','u','s','t','r','y',' ','c','o','n','t','a','c','t','l','e','s','s',' ','i','n', 0xFE
};

void PassThrough_Demo();
uint16_t demo_init();
void     demo_deinit();

uint8_t* last_intterupt_id; //&rf_present or &pt_interrupt
uint8_t rf_present;

void PININT_IRQ_HANDLER(void)
{
	//BOOL status = true;
	Chip_PININT_ClearIntStatus(LPC_PININT, PININTCH(ED_PIN_INDEX));
	i2c_rf_interrupt = false;
	rf_i2c_interrupt = false;
	rf_present = false;
	if(last_intterupt_id == &rf_present){
		rf_present = true;
	} else {
		rf_present = true;
		if(g_Direction == i2c_rf)
			i2c_rf_interrupt = true;
		else
			rf_i2c_interrupt = true;
	}
}
void Config_ED_NFC_Field()
{
	if(ntag5_WriteRegister(NTAG_MEM_OFFSET_ED_CONF_REG, 0x00FF, 0b0001)) {
		DEBUGOUT("Read aEDConfReg NOT success\r\n");
		return;
	}

	last_intterupt_id = &rf_present;
}
void Config_ED_Last_Byte_Written()
{
	if(ntag5_WriteRegister(NTAG_MEM_OFFSET_ED_CONF_REG, 0x00FF, 0b0100)) {
		DEBUGOUT("Read aEDConfReg NOT success\r\n");
		return;
	}

	last_intterupt_id = &rf_i2c_interrupt;
}
void Config_ED_Last_Byte_Read()
{
	if(ntag5_WriteRegister(NTAG_MEM_OFFSET_ED_CONF_REG, 0x00FF, 0b0011)) {
		DEBUGOUT("Read aEDConfReg NOT success\r\n");
		return;
	}

	last_intterupt_id = &i2c_rf_interrupt;
}

uint8_t Wait_For_ED_Low() {
	/* Go to deep sleep state - will wake up automatically on interrupt */
	if(Chip_GPIO_ReadPortBit(LPC_GPIO, ED_PIN_PORT, ED_PIN) != 0)
#ifdef DEBUG
		Chip_PMU_SleepState(LPC_PMU);
#else
		Chip_PMU_DeepSleepState(LPC_PMU);
#endif
	return false;
}


#ifdef USE_TIMER
#define TICKRATE_HZ 1 /* 1Hz */
uint8_t timer_out = 0;
void TIMER32_0_IRQHandler(void)
{
	if (Chip_TIMER_MatchPending(LPC_TIMER32_0, 1)) {
		Chip_TIMER_ClearMatch(LPC_TIMER32_0, 1);
		if(timer_out) {
			Chip_GPIO_SetPortOutHigh(LPC_GPIO, 0, (1UL << 5));
			Board_LED_Set(LED3_RED, true);
		} else {
			Chip_GPIO_SetPortOutLow(LPC_GPIO, 0, (1UL << 5));
			Board_LED_Set(LED3_RED, false);
		}
		timer_out = !timer_out;
	}
}
#endif /* USE_TIMER */



void ERROR_TRAP() {
	while(1) {
		Board_LED_Set(RGB_LED_BLUE, true);
#ifdef DEBUG
			Chip_PMU_SleepState(LPC_PMU);
#else
			Chip_PMU_DeepSleepState(LPC_PMU);
#endif
	}
}




/*****************************************************************************
 * Demo program
 ****************************************************************************/

/**
 * @brief	Main program body
 * @return	int
 */
int main(void)
{
	//someDelay(); /* Led the cap load */
	if(demo_init() == false) {
		PassThrough_Demo();
		demo_deinit();
	}
	return 0;
}


void PassThrough_Demo(void* pDta)
{
    uint8_t  error_status;
    uint16_t value = 0;

	while(1)
	{
		if(rf_present == true) /* Check if NFC field is present (this variable is set in the Event Detection interrupt) */
		{
			if (g_Direction == rf_i2c)
			{
				/* Check if the tag is already configured in this mode. */
				if(!tag_configured)
				{
					rf_sram_written = false; /* SRAM hasn't been written via RF yet */

					/* Check if VCC and NFC are OK. Also Check that SRAM_DATA_READY == 0,
					 *  this indicates SRAM is ready to be written via RF */
					error_status =  ntag5_ReadRegister(NTAG_MEM_OFFSET_TAG_STATUS_REG, &value);
					if(!error_status && ((value & 0x0003) == 0x0003) && !((value & 0x0020) == 0x0020))
					{
						/*Enable the Pass through mode in session registers. set the direction as RF to I2C
						 *  and activate SRAM_MAPPING*/
						error_status = ntag5_WriteRegister(NTAG_MEM_OFFSET_TAG_CONFIG0_REG,0xFF00, 0x0200);
						if(error_status) continue;
						error_status = ntag5_WriteRegister(NTAG_MEM_OFFSET_TAG_CONFIG0_REG,0x0C00, 0x0800);
						if(error_status) continue;
						error_status = ntag5_WriteRegister(NTAG_MEM_OFFSET_TAG_CONFIG0_REG,0x0100, 0x0100);
						if(error_status) continue;

						/* Check if PT direction is correctly set in the TAG STATUS REG */
						error_status = ntag5_ReadRegister(NTAG_MEM_OFFSET_TAG_STATUS_REG, &value);
						if(!error_status && ((value & 0x0007) == 0x0007)) tag_configured = true;
						else continue;

						Board_LED_Set(LED1_RED, false); /* Operation LED in KW41Z is off, still no data has been read via I2C */

						rf_i2c_interrupt = false;

						/* Configure ED pin in NTAG5 to detect when the last SRAM byte has been written via RF */
						Config_ED_Last_Byte_Written();
						Wait_For_ED_Low();
					}
				}
				else
				{
					if(rf_i2c_interrupt)
					{
						/* Check if NFC and VCC are OK. Also check that SRAM_DATA_READY == 1 so the data is
						 *  ready to be read via I2C */
						error_status = ntag5_ReadRegister(NTAG_MEM_OFFSET_TAG_STATUS_REG, &value);
						if(!error_status && ((value & 0x0027) == 0x0027))
						{
							rf_sram_written = true; /* This confirms that the SRAM has been written via RF */
						}
						else{
							rf_sram_written = false;
							Board_LED_Set(LED1_RED, false);
						}

						if(rf_sram_written && (read_loop_counter<5))
						{
							/* Check if SRAM_DATA_READY is set and start reading the SRAM, also check if VCC and NFC are OK*/
							error_status =  ntag5_ReadRegister(NTAG_MEM_OFFSET_TAG_STATUS_REG, &value);
							if(!error_status && ((value & 0x0023) == 0x0023))
							{
								/* Read SRAM */
								error_status = ntag5_ReadSRAM();
								if(error_status) continue;

								read_loop_counter++;

								if(read_loop_counter<5)
								{
									rf_i2c_interrupt = false;

									Config_ED_Last_Byte_Written();
									Wait_For_ED_Low();
								}

								/* SRAM read via I2C is indicated with LED red ON*/
								Board_LED_Set(LED1_RED, true);
							}
						}
						else if(read_loop_counter == 5) /* When the application loops for 5th time automatically change PT direction */
						{
							g_Direction = i2c_rf;
							Board_LED_Set(LED3_RED, true);
							Board_LED_Set(LED2_RED, false);

							tag_configured = false;

							/* Reset all counters and interrupt indicators */
							Board_LED_Set(LED1_RED, false);

							read_loop_counter = 0;
							write_loop_counter = 0;

							rf_i2c_interrupt = false;
							i2c_rf_interrupt = false;
						}
					}
				}
			}
			else if (g_Direction == i2c_rf)
			{
				/* Check if the tag is already configured in this mode*/
				if(!tag_configured)
				{
					rf_sram_read = false;

					/* Check if VCC and NFC are OK*/
					error_status =  ntag5_ReadRegister(NTAG_MEM_OFFSET_TAG_STATUS_REG, &value);
					if(!error_status && ((value & 0x0003) == 0x0003))
					{
						/*Enable the Pass through mode in session registers. set the direction
						 *  as I2C to RF and activate SRAM_MAPPING*/
						error_status = ntag5_WriteRegister(NTAG_MEM_OFFSET_TAG_CONFIG0_REG,0xFF00, 0x0200);
						if(error_status) continue;
						error_status = ntag5_WriteRegister(NTAG_MEM_OFFSET_TAG_CONFIG0_REG,0x0C00, 0x0800);
						if(error_status) continue;
						error_status = ntag5_WriteRegister(NTAG_MEM_OFFSET_TAG_CONFIG0_REG,0x0100, 0x0000);
						if(error_status) continue;

						/* Check that NFC and VCC are OK. Check that PT mode is correctly configured */
						error_status = ntag5_ReadRegister(NTAG_MEM_OFFSET_TAG_STATUS_REG, &value);
						if(!error_status && ((value & 0x0003) == 0x0003)) tag_configured = true;
						else continue;

						/* Write the SRAM for the first time when PT mode is configured*/
						error_status = ntag5_WriteSRAM(&gSRamDataFixed[0]);
						if(error_status) continue;
						else write_loop_counter++;

						/* SRAM written via I2C indicated with LED red ON*/
						Board_LED_Set(LED1_RED, true);

						/* Configure the ED pin to trigger the interrupt when the last byte has been read via RF.*/
						i2c_rf_interrupt = false;
						Config_ED_Last_Byte_Read();
						Wait_For_ED_Low();
					}
				}
				else
				{
					if(i2c_rf_interrupt)
					{
						/* Before start writing check that SRAM_DATA_REAY == 0, which means
						 *  the data has been read via RF and is ready
						 * to be written via I2C. Also check that NFC and VCC are ok.*/
						error_status = ntag5_ReadRegister(NTAG_MEM_OFFSET_TAG_STATUS_REG, &value);
						if(!error_status && ((value & 0x0003) == 0x0003) && !((value & 0x0020) == 0x0020))
							rf_sram_read = true;
						else{
							rf_sram_read = false;
							Board_LED_Set(LED1_RED, false);
						}

						if(rf_sram_read && write_loop_counter < 5)
						{
							/* Write SRAM via I2C.*/
							error_status = ntag5_WriteSRAM(&gSRamDataFixed[0]);
							if(error_status) continue;

							write_loop_counter++;

							if(write_loop_counter < 5)
							{
								/* Configure the ED pin to trigger the interrupt when the last byte has been read via RF.*/
								i2c_rf_interrupt = false;
								Config_ED_Last_Byte_Read();
								Wait_For_ED_Low();
							}

							/* SRAM written via I2C indicated with LED red ON*/
							Board_LED_Set(LED1_RED, true);
						}
						else if(write_loop_counter == 5) /* When the application loops for 5th time automatically
						 change PT direction */
						{
							g_Direction = rf_i2c;
							Board_LED_Set(LED3_RED, false);
							Board_LED_Set(LED2_RED, true);

							tag_configured = false;

							/* Reset all counters and interrupt indicators */
							Board_LED_Set(LED1_RED, false);

							read_loop_counter = 0;
							write_loop_counter = 0;

							i2c_rf_interrupt = false;
							rf_i2c_interrupt = false;
						}
					}
				}
			}
		}
		else
		{
			/* Set the RF field present interrupt to detect when the phone has tapped the antenna.*/
			Config_ED_NFC_Field();
			Wait_For_ED_Low();
		}
	}
}


// sets up the Board so that the NTAG can be used
void HW_setup_Board_for_use_with_NTAG() {

	// Disable clock for  USB RAM, USB, and
	LPC_SYSCTL->SYSAHBCLKCTRL &= ~(
//			(1 <<  0) | /* SYS */
//			(1 <<  1) | /* ROM */
//			(1 <<  2) | /* RAM0 */
//			(1 <<  3) | /* FLASHREG */
//			(1 <<  4) | /* FLASHARRAY */
//			(1 <<  5) | /* I2C */
//			(1 <<  6) | /* GPIO */
			(1 <<  7) | /* CT16B0 */
			(1 <<  8) | /* CT16B1 */
			(1 <<  9) | /* CT32B0 */
			(1 << 10) | /* CT32B1 */
			(1 << 11) | /* SSP0 */
			(1 << 12) | /* USART */
			(1 << 13) | /* ADC */
			(1 << 14) | /* USB */
//			(1 << 15) | /* WWDT */
//			(1 << 16) | /* IOCON */
//			(1 << 17) | /*  */
			(1 << 18) | /* SSP1 */
//			(1 << 19) | /* PINT */
//			(1 << 20) | /*  */
//			(1 << 21) | /*  */
//			(1 << 22) | /*  */
//			(1 << 23) | /* GROUP0INT */
//			(1 << 24) | /* GROUP1INT */
//			(1 << 25) | /*  */
//			(1 << 26) | /* RAM1 */
//			(1 << 27) | /* USBRAM */
					0 );

	/* Power down config */
	Chip_SYSCTL_PowerDown(
		SYSCTL_POWERDOWN_IRCOUT_PD |	/*!< IRC oscillator output power down */
		SYSCTL_POWERDOWN_IRC_PD    |	/*!< IRC oscillator power-down */
//		SYSCTL_POWERDOWN_FLASH_PD  |	/*!< Flash power down */
		SYSCTL_POWERDOWN_BOD_PD    |	/*!< BOD power down */
		SYSCTL_POWERDOWN_ADC_PD    |	/*!< ADC power down */
//		SYSCTL_POWERDOWN_SYSOSC_PD |	/*!< System oscillator power down */
//		SYSCTL_POWERDOWN_WDTOSC_PD |	/*!< Watchdog oscillator power down */
		SYSCTL_POWERDOWN_SYSPLL_PD |	/*!< System PLL power down */
		SYSCTL_POWERDOWN_USBPLL_PD |	/*!< USB PLL power-down */
		SYSCTL_POWERDOWN_USBPAD_PD |    /*!< USB transceiver power-down */
	0);


	/* Wake up power down config */
	Chip_SYSCTL_SetWakeup(
		SYSCTL_SLPWAKE_IRCOUT_PD |	/*!< IRC oscillator output wake-up configuration */
		SYSCTL_SLPWAKE_IRC_PD    |	/*!< IRC oscillator power-down wake-up configuration */
//		SYSCTL_SLPWAKE_FLASH_PD  |	/*!< Flash wake-up configuration */
		SYSCTL_SLPWAKE_BOD_PD    |	/*!< BOD wake-up configuration */
		SYSCTL_SLPWAKE_ADC_PD    |	/*!< ADC wake-up configuration */
//		SYSCTL_SLPWAKE_SYSOSC_PD |	/*!< System oscillator wake-up configuration */
//		SYSCTL_SLPWAKE_WDTOSC_PD |	/*!< Watchdog oscillator wake-up configuration */
//		SYSCTL_SLPWAKE_SYSPLL_PD |	/*!< System PLL wake-up configuration */
		SYSCTL_SLPWAKE_USBPLL_PD |	/*!< USB PLL wake-up configuration */
		SYSCTL_SLPWAKE_USBPAD_PD |	/*!< USB transceiver wake-up configuration */
	0);

	/* Deep sleep power down config */
	Chip_SYSCTL_SetDeepSleepPD(SYSCTL_DEEPSLP_BOD_PD);
//		SYSCTL_DEEPSLP_BOD_PD    /*!< BOD power-down control in Deep-sleep mode, powered down */
//		SYSCTL_DEEPSLP_WDTOSC_PD /*!< Watchdog oscillator power control in Deep-sleep, powered down */

}



uint16_t demo_init() {

	//###########################   BOARD  ############################
	/* Get system clock */
	SystemCoreClockUpdate();
//	volatile int clock = SystemCoreClock/1000000; /* Get system core clock in MHz */

	/* Init GPIO and LEDs */
	Board_Init();
	/* Power down not needed components and disable unused peripheral clocks */
	HW_setup_Board_for_use_with_NTAG();

	/* Configure ED pin as input pin */
	Chip_GPIO_SetPinDIRInput(LPC_GPIO, ED_PIN_PORT, ED_PIN);
	/* Configure ED pin with pullup */
	Chip_IOCON_PinMuxSet(LPC_IOCON, ED_PIN_PORT, ED_PIN, (IOCON_FUNC0 | IOCON_MODE_PULLUP | IOCON_DIGMODE_EN));


	//########################### INTERRUPT ############################
	/* Enable PININT clock */
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_PINT);
	/* Configure interrupt channel for the GPIO pin in SysCon block */
	Chip_SYSCTL_SetPinInterrupt(ED_PIN_INDEX, ED_PIN_PORT, ED_PIN);
	/* Configure channel interrupt as edge sensitive and falling edge interrupt */
	Chip_PININT_SetPinModeEdge(LPC_PININT, PININTCH(ED_PIN_INDEX));
	Chip_PININT_EnableIntLow(LPC_PININT, PININTCH(ED_PIN_INDEX));

	/* Configure channel as wake up interrupt in SysCon block */
	Chip_SYSCTL_EnablePINTWakeup(ED_PIN_INDEX);

	/* Enable all clocks, even those turned off at wake power up */
	Chip_SYSCTL_SetWakeup(~(SYSCTL_SLPWAKE_IRCOUT_PD | SYSCTL_SLPWAKE_IRC_PD |
	SYSCTL_SLPWAKE_FLASH_PD | SYSCTL_SLPWAKE_SYSOSC_PD | SYSCTL_SLPWAKE_SYSOSC_PD | SYSCTL_SLPWAKE_SYSPLL_PD));

	/* Enable interrupt in the NVIC */
	NVIC_EnableIRQ(PININT_NVIC_NAME);


#ifdef USE_TIMER
	//########################### TIMER ############################
	/* Enable timer 1 clock */
	Chip_TIMER_Init(LPC_TIMER32_0);
	/* Timer rate is system clock rate */
	uint32_t timerFreq = Chip_Clock_GetSystemClockRate();

	/* Timer setup for match and interrupt at TICKRATE_HZ */
	Chip_TIMER_Reset(LPC_TIMER32_0);
	Chip_TIMER_MatchEnableInt(LPC_TIMER32_0, 1);
	Chip_TIMER_SetMatch(LPC_TIMER32_0, 1, (timerFreq / TICKRATE_HZ));
	Chip_TIMER_ResetOnMatchEnable(LPC_TIMER32_0, 1);
	Chip_TIMER_Enable(LPC_TIMER32_0);

	/* Enable timer interrupt */
	NVIC_ClearPendingIRQ(TIMER_32_0_IRQn);
	NVIC_EnableIRQ(TIMER_32_0_IRQn);
#endif /* USE_TIMER */


	//###########################   I2C   ############################

	/* Initialize I2C */
	Init_I2C_PinMux();
	Chip_I2C_Init(DEFAULT_I2C);
	Chip_I2C_SetClockRate(DEFAULT_I2C, I2C_DEFAULT_SPEED);
	/* Set default mode to interrupt */
	i2c_set_mode(DEFAULT_I2C, 0);

	/* Check for NTAG5 on I2C lines */
	if(i2c_probe_slave(DEFAULT_I2C, NTAG5_I2C_ADDR)) {
		ERROR_TRAP();
	}



	/* Set arbiter mode to pass_trough */
	uint8_t aConfigReg[4];
	if(ntag5_ReadRegister(NTAG_MEM_OFFSET_TAG_CONFIG0_REG, (uint16_t*)(&aConfigReg[0]))) {
		DEBUGOUT("Read config0Reg NOT success\r\n");
		ERROR_TRAP();
	}
	if((aConfigReg[1] & 0b1100) != 0b1000) {
		if(ntag5_WriteRegister(NTAG_MEM_OFFSET_TAG_CONFIG0_REG, ((uint16_t)0b1100)<<8, ((uint16_t)0b1000)<<8)) {
			DEBUGOUT("Write config0Reg NOT success\r\n");
			ERROR_TRAP();
		}
	}

	return false;
}

void demo_deinit() {
	Chip_I2C_DeInit(DEFAULT_I2C);
}
