/*
 * @brief I2C example.
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include <stdlib.h>
#include <string.h>
#include "board.h"
#include <sysctl_11xx.h>

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/


#define LED0_RED      0 /* Red left: near NXPLOGO */
#define LED1_RED      1 /* Red second */
#define LED2_RED      2 /* Red third  */
#define LED3_RED      3 /* Red fourth */
#define RGB_LED_RED   4 /* Red   RGB  */
#define RGB_LED_GREEN 5/* Green RGB  */
#define RGB_LED_BLUE  6/* Blue  RGB  */


#define INT0 0
#define INT1 1
// sets up the Board so that the NTAG can be used
void HW_setup_Board_for_use_with_NTAG() {

	// Disable clock for  USB RAM, USB, and TODO: SSP0 to minimize power in energy harvesting mode
	LPC_SYSCTL->SYSAHBCLKCTRL &= ~(
//			(1 <<  0) | /* SYS */
//			(1 <<  1) | /* ROM */
//			(1 <<  2) | /* RAM0 */
//			(1 <<  3) | /* FLASHREG */
//			(1 <<  4) | /* FLASHARRAY */
//			(1 <<  5) | /* I2C */
//			(1 <<  6) | /* GPIO */
			(1 <<  7) | /* CT16B0 */
			(1 <<  8) | /* CT16B1 */
			(1 <<  9) | /* CT32B0 */
			(1 << 10) | /* CT32B1 */
			(1 << 11) | /* SSP0 */
			(1 << 12) | /* USART */
			(1 << 13) | /* ADC */
			(1 << 14) | /* USB */
//			(1 << 15) | /* WWDT */
//			(1 << 16) | /* IOCON */
//			(1 << 17) | /*  */
			(1 << 18) | /* SSP1 */
//			(1 << 19) | /* PINT */
//			(1 << 20) | /*  */
//			(1 << 21) | /*  */
//			(1 << 22) | /*  */
//			(1 << 23) | /* GROUP0INT */
//			(1 << 24) | /* GROUP1INT */
//			(1 << 25) | /*  */
//			(1 << 26) | /* RAM1 */
//			(1 << 27) | /* USBRAM */
					0 );

	/* Power down config */
	Chip_SYSCTL_PowerDown(
		SYSCTL_POWERDOWN_IRCOUT_PD |	/*!< IRC oscillator output power down */
		SYSCTL_POWERDOWN_IRC_PD    |	/*!< IRC oscillator power-down */
//		SYSCTL_POWERDOWN_FLASH_PD  |	/*!< Flash power down */
		SYSCTL_POWERDOWN_BOD_PD    |	/*!< BOD power down */
		SYSCTL_POWERDOWN_ADC_PD    |	/*!< ADC power down */
//		SYSCTL_POWERDOWN_SYSOSC_PD |	/*!< System oscillator power down */
//		SYSCTL_POWERDOWN_WDTOSC_PD |	/*!< Watchdog oscillator power down */
		SYSCTL_POWERDOWN_SYSPLL_PD |	/*!< System PLL power down */
		SYSCTL_POWERDOWN_USBPLL_PD |	/*!< USB PLL power-down */
		SYSCTL_POWERDOWN_USBPAD_PD |    /*!< USB transceiver power-down */
	0);


	/* Wake up power down config */
	Chip_SYSCTL_SetWakeup(
		SYSCTL_SLPWAKE_IRCOUT_PD |	/*!< IRC oscillator output wake-up configuration */
		SYSCTL_SLPWAKE_IRC_PD    |	/*!< IRC oscillator power-down wake-up configuration */
//		SYSCTL_SLPWAKE_FLASH_PD  |	/*!< Flash wake-up configuration */
		SYSCTL_SLPWAKE_BOD_PD    |	/*!< BOD wake-up configuration */
		SYSCTL_SLPWAKE_ADC_PD    |	/*!< ADC wake-up configuration */
//		SYSCTL_SLPWAKE_SYSOSC_PD |	/*!< System oscillator wake-up configuration */
//		SYSCTL_SLPWAKE_WDTOSC_PD |	/*!< Watchdog oscillator wake-up configuration */
//		SYSCTL_SLPWAKE_SYSPLL_PD |	/*!< System PLL wake-up configuration */
		SYSCTL_SLPWAKE_USBPLL_PD |	/*!< USB PLL wake-up configuration */
		SYSCTL_SLPWAKE_USBPAD_PD |	/*!< USB transceiver wake-up configuration */
	0);

	/* Deep sleep power down config */
	Chip_SYSCTL_SetDeepSleepPD(SYSCTL_DEEPSLP_BOD_PD);
//		SYSCTL_DEEPSLP_BOD_PD    /*!< BOD power-down control in Deep-sleep mode, powered down */
//		SYSCTL_DEEPSLP_WDTOSC_PD /*!< Watchdog oscillator power control in Deep-sleep, powered down */

}


void FLEX_INT0_IRQHandler(void)
{
    Chip_PININT_ClearIntStatus(LPC_PININT, PININTCH(INT0));
}
void FLEX_INT1_IRQHandler(void)
{
    Chip_PININT_ClearIntStatus(LPC_PININT, PININTCH(INT1));
}



/**
 * @brief	Main program body
 * @return	int
 */
int main(void)
{
	/* Get system clock */
	SystemCoreClockUpdate();
	//volatile int clock = SystemCoreClock/1000000; /* Get system core clock in MHz */

	/* Init GPIO and LEDs */
	Board_Init();
	/* Power down not needed components and disable unused peripheral clocks */
	HW_setup_Board_for_use_with_NTAG();

	/* Configure P0_4/SCL as GPIO-input pin */
	Chip_GPIO_SetPinDIRInput(LPC_GPIO, 0, 4);
	Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 4, (IOCON_FUNC0));

	/* Configure P0_5/SDA as GPIO-input pin */
	Chip_GPIO_SetPinDIRInput(LPC_GPIO, 0, 5);
	Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 5, (IOCON_FUNC0));

    //########################### INTERRUPT ############################
    /* Enable PININT clock */
    Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_PINT);
    /* Configure interrupt channel for the GPIO pin in SysCon block */
    Chip_SYSCTL_SetPinInterrupt(INT0, 0, 4);
    Chip_SYSCTL_SetPinInterrupt(INT1, 0, 5);

    /* Configure channels interrupt as edge sensitive and falling edge interrupt */
    Chip_PININT_SetPinModeEdge(LPC_PININT, PININTCH(INT0));
    Chip_PININT_EnableIntHigh (LPC_PININT, PININTCH(INT0));
    Chip_PININT_EnableIntLow  (LPC_PININT, PININTCH(INT0));

    Chip_PININT_SetPinModeEdge(LPC_PININT, PININTCH(INT1));
    Chip_PININT_EnableIntHigh (LPC_PININT, PININTCH(INT1));
    Chip_PININT_EnableIntLow  (LPC_PININT, PININTCH(INT1));

    /* Configure channels as wake up interrupt in SysCon block */
    Chip_SYSCTL_EnablePINTWakeup(0);
    Chip_SYSCTL_EnablePINTWakeup(1);

    /* Enable interrupts in the NVIC */
    NVIC_EnableIRQ(PIN_INT0_IRQn);
    NVIC_EnableIRQ(PIN_INT1_IRQn);


    /* Loop */
	while(1) {
		/* Wait for interrupt */
//        Chip_PMU_SleepState(LPC_PMU);

        /* Write out pin states */
		uint32_t port = Chip_GPIO_ReadValue(LPC_GPIO, 0);
		uint32_t p0_4_scl = port & (1UL << 4);
		uint32_t p0_5_sda = port & (1UL << 5);

		Board_LED_Set(RGB_LED_GREEN, p0_4_scl ? true : false);
		Board_LED_Set(LED0_RED, p0_5_sda ? true : false);
	}

	return 0;
}

